Package: MPAC
Title: Multi-omic Pathway Analysis of Cells
Version: 1.6.0
Authors@R: c(
    person(given="Peng", family="Liu", 
           email="pliu55.wisc+bioconductor@gmail.com", 
           role = c("aut", "cre"), comment=c(ORCID="0000-0001-5655-2259")),
    person(given="Paul",    family="Ahlquist", role = c("aut")),
    person(given="Irene",   family="Ong",      role = c("aut")),
    person(given="Anthony", family="Gitter",   role = c("aut")))
Description: Multi-omic Pathway Analysis of Cells (MPAC), integrates 
    multi-omic data for understanding cellular mechanisms. It  
    predicts novel patient groups with distinct pathway profiles 
    as well as identifying key pathway proteins with potential clinical 
    associations. From CNA and RNA-seq data, it determines genes’ DNA and RNA
    states (i.e., repressed, normal, or activated), which serve as the input 
    for PARADIGM to calculate Inferred Pathway Levels (IPLs). It also permutes
    DNA and RNA states to create a background distribution to filter IPLs as a 
    way to remove events observed by chance. It provides multiple methods for
    downstream analysis and visualization.
License: GPL-3
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
Depends: R (>= 4.4.0)
URL: https://github.com/pliu55/MPAC
BugReports: https://github.com/pliu55/MPAC/issues
Imports: data.table (>= 1.14.2), SummarizedExperiment (>= 1.30.2),
        BiocParallel (>= 1.28.3), fitdistrplus (>= 1.1), igraph (>=
        1.4.3), BiocSingular (>= 1.10.0), S4Vectors (>= 0.32.3),
        SingleCellExperiment (>= 1.16.0), bluster (>= 1.4.0), fgsea (>=
        1.20.0), scran (>= 1.22.1), ComplexHeatmap (>= 2.16.0),
        circlize (>= 0.4.16), scales (>= 1.3.0), stringr (>= 1.5.1),
        viridis (>= 0.6.5), ggplot2 (>= 3.5.1), ggraph (>= 2.2.1),
        survival (>= 3.7), survminer (>= 0.4.9), grid, stats
Suggests: rmarkdown, knitr, svglite, bookdown(>= 0.34), testthat (>=
        3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
biocViews: Software, Technology, Sequencing, RNASeq, Survival,
        Clustering, ImmunoOncology
SystemsRequirements: The `runPrd()` function requires an external
        software named PARADIGM. For details, please see the 'Required
        external software' section in vignette's 'Run PARADIGM:
        runPrd()'.
git_url: https://git.bioconductor.org/packages/MPAC
git_branch: RELEASE_3_23
git_last_commit: 2a03ee8
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:24:33 UTC; biocbuild
Author: Peng Liu [aut, cre] (ORCID: <https://orcid.org/0000-0001-5655-2259>),
  Paul Ahlquist [aut],
  Irene Ong [aut],
  Anthony Gitter [aut]
Maintainer: Peng Liu <pliu55.wisc+bioconductor@gmail.com>

# Generated by roxygen2: do not edit by hand

export(clSamp)
export(colPermIPL)
export(colRealIPL)
export(conMtf)
export(fltByPerm)
export(getSignifOvrOnCl)
export(ovrGMT)
export(pltConMtf)
export(pltMtfPrtIPL)
export(pltNeiStt)
export(pltOvrHm)
export(pltSttKM)
export(ppCnInp)
export(ppPermInp)
export(ppRealInp)
export(ppRnaInp)
export(ppRunPrd)
export(runPermPrd)
export(runPrd)
export(subNtw)
import(ComplexHeatmap)
import(SingleCellExperiment)
import(data.table)
import(ggplot2)
import(ggraph)
import(igraph)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,SnowParam)
importFrom(BiocParallel,bplapply)
importFrom(BiocSingular,RandomParam)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,metadata)
importFrom(SummarizedExperiment,"assays<-")
importFrom(SummarizedExperiment,"rowData<-")
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assays)
importFrom(bluster,NNGraphParam)
importFrom(circlize,colorRamp2)
importFrom(fgsea,gmtPathways)
importFrom(fitdistrplus,mgedist)
importFrom(grid,gpar)
importFrom(grid,unit)
importFrom(scales,percent)
importFrom(scran,clusterCells)
importFrom(scran,denoisePCA)
importFrom(scran,getTopHVGs)
importFrom(scran,modelGeneVar)
importFrom(stats,fisher.test)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,p.adjust)
importFrom(stringr,str_wrap)
importFrom(survival,Surv)
importFrom(survival,survfit)
importFrom(survminer,ggsurvplot)
importFrom(viridis,cividis)

[![bioc](http://www.bioconductor.org/shields/years-in-bioc/MPAC.svg)](http://bioconductor.org/packages/MPAC) [![DOI](https://zenodo.org/badge/664257440.svg)](https://zenodo.org/doi/10.5281/zenodo.10805479)

MPAC: inferring pathway activities from multi-omic data
===========================================

* * *

Table of Contents
-----------------

* [Introduction](#introduction)
* [Installation](#installation)
* [Shiny App](#shinyapp)
* [Software Dependency](#softwaredependency)
* [Reference](#reference)
* [Contact](#contact)
* [License](#license)

* * *

## <a name='introduction'></a> Introduction

Multi-omic Pathway Analysis of Cells (__MPAC__) is an __R__ package that 
interprets multi-omic data through the prior knowledge of biological pathways.
The workflow of MPAC contains several steps, which are shown in 
the figure below.  MPAC has a
[vignette](https://bioconductor.org/packages/devel/bioc/vignettes/MPAC/inst/doc/MPAC.html) that describes each function in details.

<p align='center'>
    <img src="vignettes/workflow.jpg" width="800">
</p>

## <a name='installation'></a> Installation

### From GitHub

Start __R__ and enter: 

```r
devtools::install_github('pliu55/MPAC')
```

### From Bioconductor

Start __R__ and enter:

```r
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

# The following initializes usage of Bioc devel
BiocManager::install(version='devel')

BiocManager::install("MPAC")
```


## <a name="shinyapp"></a> Shiny App

To explore key results reported in the MPAC manuscript as well as associated
results please check out
[this R Shiny app](https://github.com/pliu55/MPAC_Shiny).


## <a name="softwaredependency"></a> Software Dependency

The `runPrd()` function requires an external software named 
[PARADIGM](https://doi.org/10.1093/bioinformatics/btq182),
which is only available for 
[Linux](
https://github.com/sng87/paradigm-scripts/tree/master/public/exe/LINUX) and 
[MacOS](
https://github.com/sng87/paradigm-scripts/tree/master/public/exe/MACOSX).
For details, please see the __Required external software__ section in vignette's __Run PARADIGM: runPrd()__.


## <a name="reference"></a> Reference

### Manuscript

__MPAC: a computational framework for inferring cancer pathway activities from multi-omic data__. Peng Liu, David Page, Paul Ahlquist, Irene M. Ong*, Anthony Gitter*. _Bioinformatics_, Volume 41, Issue 10, October 2025, btaf490, [https://doi.org/10.1093/bioinformatics/btaf490](https://doi.org/10.1093/bioinformatics/btaf490), *co-corresponding author


## <a name="contact"></a> Contact

Got a question? Please report it at the [issues tab](https://github.com/pliu55/MPAC/issues) in this repository.

## <a name="license"></a> License

MPAC is licensed under the [GNU General Public License v3](LICENSE.md).

#' @title  Plot a Kaplan-Meier curve for samples stratified by given
#'         protein(s)' pathway states
#'
#' @param  cdrmat   A matrix containing survival data with rows as patient
#'                  samples and columns as survival event and time.
#'
#' @param  event  The column name in `cdrmat` to indicate survival event.
#'                Default: 'OS'.
#'
#' @param  time   The column name in `cdrmat` to indicate survival time.
#'                Default: 'OS_days'.
#'
#' @param  proteins  Rowname(s) in `fltmat`. Its/their pathway states will be
#'                   used to stratify patient samples. Default: all proteins
#'                   in `fltmat` will be used.
#'
#' @param strat_func  A function applied on protein(s) pathway states to
#'                    stratify patient samples. Available options: '>0', '<0',
#'                    Default: '>0', i.e., IPL >0 vs. the rest.
#'
#' @inheritParams pltNeiStt
#'
#' @return  A Kaplan-Meier plot
#'
#' @examples
#'
#' cdrmat <- system.file('extdata/pltSttKM/cdr.rds',package='MPAC') |> readRDS()
#' fltmat <- system.file('extdata/pltSttKM/ipl.rds',package='MPAC') |> readRDS()
#'
#' pltSttKM(cdrmat, fltmat, event='OS', time='OS_days',
#'          proteins=c('CD247', 'FASLG'))
#'
#' @export
#'
#'
#' @import ggplot2
#' @importFrom survival  Surv survfit
#' @importFrom survminer ggsurvplot
#' @importFrom stringr   str_wrap
#'
pltSttKM <- function(cdrmat, fltmat, event='OS', time='OS_days', proteins=NULL,
                     strat_func='>0'){

    if ( is.null(proteins) ) {
        proteins = rownames(fltmat)
    }

    pats = intersect(rownames(cdrmat), colnames(fltmat))
    cdrdt = cdrmat[pats, c(event, time)] |>
            as.data.table(keep.rownames='pat') |>
            setnames(c(event, time), c('event', 'time'))

    sttmat = fltmat[proteins, pats, drop=FALSE] |> sign()

    stt_sums = NULL
    if (strat_func == '>0') {
        stt_sums = colSums(sttmat)
    } else if ( strat_func == '<0') {
        stt_sums = colSums(sttmat) * (-1)
    } else {
        paste0("strat_func: ", strat_func, " not recognized\n",
               "It can only be >0 or <0\n") |> stop()
    }
    sel_pats = names(stt_sums[ stt_sums == length(proteins)])
    if ( length(sel_pats) == 0 ) {
        paste0('no sample for ', strat_func, " group\n") |> stop()
    }

    sel_lab = paste0(proteins, strat_func) |> paste(collapse=' & ') |>
              paste0(' (n=', length(sel_pats), ')') |> str_wrap(width=30)
    other_lab = paste0('others (n=', ncol(fltmat) - length(sel_pats), ')')

    grp = pat = NULL
    cdrdt[, grp := ifelse(pat %in% sel_pats, sel_lab, other_lab) |>
                   factor(levels=c(sel_lab, other_lab))]

    srByStt(cdrdt, event, time)
}

srByStt <- function(cdrdt, event, time) {
    frm = Surv(time, event) ~ grp
    MAGENTA = '#CC79A7'
    GREY    = '#999999'
    p = ggsurvplot(
        fit = do.call(survfit, args=list(formula=frm, data=cdrdt)),
        data = cdrdt,
        palette = c(MAGENTA, GREY),
        pval = TRUE,
        pval.size = 3,
        pval.coord = c(0.1, 0.05),
        xlab = time,
        ylab = paste0(event, ' probability'),
        legend = 'top',
        legend.title = '',
        title = '',
        ggtheme = getSrTM()
    )$plot + guides(color=guide_legend(nrow=2))

    return(p)
}

getSrTM <- function() {
    FONT_SIZE = 9

    PAPER_THEME = ggplot2::theme(
        legend.title      = ggplot2::element_text(size=FONT_SIZE),
        legend.text       = ggplot2::element_text(size=FONT_SIZE),
        axis.title        = ggplot2::element_text(size=FONT_SIZE),
        axis.text.x       = ggplot2::element_text(size=FONT_SIZE),
        axis.text.y       = ggplot2::element_text(size=FONT_SIZE),
        plot.title        = ggplot2::element_text(size=FONT_SIZE, hjust=0.5),
        legend.margin     = ggplot2::margin(b=-0.3, l=-0.3, unit='cm'),
        legend.key.size   = ggplot2::unit(0.35, 'cm'),
        panel.background  = ggplot2::element_rect(fill='transparent', color=NA),
        plot.background   = ggplot2::element_rect(fill='transparent', color=NA),
        legend.background = ggplot2::element_rect(fill='transparent', color=NA)
    )

    tm = theme_classic() + theme(aspect.ratio = 1) + PAPER_THEME

    return(tm)
}

main <- function() {
    testPltSttKM()
}

testPltSttKM <- function() {
    cdrmat = system.file('extdata/pltSttKM/cdr.rds', package='MPAC') |>readRDS()
    fltmat = system.file('extdata/pltSttKM/ipl.rds', package='MPAC') |>readRDS()

    prts = c('CD247', 'FASLG')

    test_that('testPltSttKM', {
        pltSttKM(cdrmat, fltmat, event='OS', time='OS_days', proteins=prts) |>
        print() |>
        expect_no_error()
    })

    fpdf = 'Rplots.pdf'
    if ( file.exists(fpdf) ) file.remove(fpdf)
}

main()

#' @title  Find consensus pathway motifs from a list of pathways
#'
#' @param subntwl  A list of igraph objects representing input pathways from
#'                 different samples. It is the output from `subNtw()`
#'
#' @param min_mtf_n_nodes  Number of minimum nodes in a motif. Default: 5
#'
#' @inheritParams ovrGMT
#'
#' @return  A list of igraph objects representing consensus pathway motifs
#'
#' @examples
#'
#' fsubntwl = system.file('extdata/conMtf/subntwl.rds', package='MPAC')
#' subntwl = readRDS(fsubntwl)
#'
#' fomic_gns = system.file('extdata/TcgaInp/inp_focal.rds', package='MPAC')
#' omic_gns = rownames(readRDS(fomic_gns))
#'
#' conMtf(subntwl, omic_gns, min_mtf_n_nodes=50)
#'
#' @export
#'
#' @import igraph
#'
conMtf <- function(subntwl, omic_genes=NULL, min_mtf_n_nodes=5) {
    ent <- n_pats <- NULL

    n_subntws <- length(subntwl)
    names(subntwl) <- paste0('samp', seq_len(n_subntws))

    con_ents <- lapply(names(subntwl), function(pat) {
        data.table(pat=pat, ent=V(subntwl[[pat]])$name)
    }) |> rbindlist() |>
    _[, list(n_pats = .N), by=ent] |> _[ n_pats == n_subntws]$ent

    conl <- induced_subgraph(subntwl[[1]], con_ents) |>
        decompose(min.vertices=min_mtf_n_nodes)

    out_conl <- NULL
    if ( is.null(omic_genes) ) {
        out_conl <- conl
    } else {
        out_conl <- Filter(function(grph) {
            length(intersect(V(grph)$name, omic_genes)) > 0
        }, conl)
    }

    return(out_conl)
}

require(data.table)
suppressMessages(require(igraph))

main <- function() {
    testConMtf()
}

testConMtf <- function() {
    fin = system.file('extdata/conMtf/subntwl.rds', package='MPAC')
    subntwl = readRDS(fin) |> lapply(function(s) upgrade_graph(s))

    ffocal = system.file('extdata/TcgaInp/inp_focal.rds', package='MPAC')
    omic_gns = readRDS(ffocal) |> rownames()

    outl = conMtf(subntwl, omic_gns, min_mtf_n_nodes=50)

    cmpl = system.file('extdata/conMtf/con_grph.rds', package='MPAC') |>
           readRDS() |> lapply(function(s) upgrade_graph(s))

    lapply(seq_len(length(outl)), testConMtfByIndi, outl, cmpl)
}

testConMtfByIndi <- function(ind, outl, cmpl) {
    out = outl[[ind]]
    cmp = cmpl[[ind]]

    test_that('testConMtf: vertex', {
        expect_identical( sort(V(out)$name), sort(V(cmp)$name) )
    })

    test_that('testConMtf: edge', {
        expect_identical(
            as_edgelist(out) |> as.data.table() |> _[order(V1, V2)],
            as_edgelist(cmp) |> as.data.table() |> _[order(V1, V2)]
        )
    })
}

main()

#' @importFrom BiocParallel SnowParam MulticoreParam
#'
getBPPARAM <- function(threads) {
    bp <- NULL
    if ( getOS() == 'Windows' ) {
        bp <- SnowParam(workers=threads, type='SOCK')
    } else {
        bp <- MulticoreParam(workers=threads)
    }
    return(bp)
}

getOS <- function() {
    os <- toupper(.Platform$OS.type)
    sysinf <- Sys.info()
    if ( !is.null(sysinf) ) {
        os <- sysinf['sysname']
        if ( os == 'Darwin' ) {
            os <- "OSX"
        } else if ( os == 'Linux' ) {
            os <- "LINUX"
        } else {
            os <- "WINDOWS"
        }
    } else {
        if ( grepl("^darwin", R.version$os, perl=TRUE) ) {
            os <- "OSX"
        } else if ( grepl("linux-gnu", R.version$os, perl=TRUE) ) {
            os <- "LINUX"
        } else {
            os <- "WINDOWS"
        }
    }

    return(os)
}

getNodeEdge <- function(fpth) {
    . <- entity <- type <- from <- to <- title <- NULL

    wordslist <- readLines(fpth) |> strsplit("\t")

    nodedt <- lapply(wordslist, function(words) {
        if (length(words) == 2) as.list(words)
    }) |> rbindlist() |> setnames( c('type', 'entity') ) |>
    _[, .(entity, type)]

    edgedt <- lapply(wordslist, function(words) {
        if (length(words) == 3) as.list(words)
    }) |> rbindlist() |> setnames( c('from', 'to', 'title') ) |>
    _[, .(from, to, title)]

    list(nodedt=nodedt, edgedt=edgedt)
}

#' @title  Collect Inferred Pathway Levels (IPLs) from PARADIGM runs on real
#'         data
#'
#' @param indir  Input folder that saves PARADIGM results. It should be set as
#'               the same as `outdir` as in `runPrd()`.
#'
#' @param sampleids  Sample IDs for which IPLs to be collected. If not provided,
#'                   all files with suffix '_ipl.txt' in `indir` will be
#'                   collected. Default: NULL.
#'
#' @param file_tag  A string of output file name tag. Default: NULL
#'
#' @return  A data.table object with columns of pathway entities and their IPLs.
#'
#' @export
#'
#' @examples
#'
#' indir = system.file('/extdata/runPrd/', package='MPAC')
#'
#' colRealIPL(indir)
#'
colRealIPL <- function(indir, sampleids=NULL, file_tag=NULL) {
    colIPL(indir, sampleids, file_tag)
}

#' @title  Collect Inferred Pathway Levels (IPLs) from PARADIGM runs on permuted
#'         data
#'
#' @inheritParams colRealIPL
#'
#' @param  n_perms  Number of permutations to collect.
#'
#' @param threads  Number of threads to run in parallel. Default: 1
#'
#' @return  A data.table object with columns of permutation index, pathway
#'          entities and their IPLs.
#'
#' @export
#'
#' @examples
#'
#' indir = system.file('/extdata/runPrd/', package='MPAC')
#' n_perms = 3
#'
#' colPermIPL(indir, n_perms)
#'
#' @importFrom BiocParallel  SnowParam bplapply
#'
colPermIPL <- function(indir, n_perms, sampleids=NULL, threads=1) {
    bp <- getBPPARAM(threads)
    bplapply(seq_len(n_perms), function(iperm) {
        ipldt <- paste0(indir, '/p', iperm, '/') |> colIPL(sampleids, iperm)
        brcs <- names(ipldt) |> setdiff('entity')
        ipldt[, iperm := iperm] |>
        _[, c('entity', 'iperm', brcs), with=FALSE]
    }, BPPARAM=bp) |> rbindlist()
}

colIPL <- function(indir, sampleids, file_tag=NULL) {
    fipls <- NULL
    suffix <- ifelse(is.null(file_tag), '_ipl.txt',
        paste0('_', file_tag, '_ipl.txt'))

    if ( is.null(sampleids) ) {
        fipls <- list.files(path=indir, pattern="*_ipl.txt", full.names=TRUE,
            recursive=FALSE)
    } else {
        fipls <- paste0(indir, '/', sampleids, suffix)
    }

    sampleid <- fipl <- NULL
    fdt <- data.table(fipl = fipls) |>
        _[, sampleid := basename(fipl) |> tstrsplit(suffix) |> _[[1]] ]

    Map(function(fin, sampleid) {
        readIPL(fin) |> _[, sampleid := sampleid]
    }, fdt$fipl, fdt$sampleid) |> rbindlist() |>
    dcast(entity ~ sampleid, value.var='ipl')
}

readIPL <- function(fin) {
    readLines(fin) |>
    lapply(function(line) {
        if ((! grepl('^> ',     line, perl=TRUE)) &
            (! grepl('__\\d\t', line, perl=TRUE)) ) {
            words <- strsplit(line, "\t")[[1]]
            list(entity=words[1], ipl=as.numeric(words[2]))
        }
    }) |> rbindlist()
}

main <- function() {
    testColRealIPL()

    testColPermIPL()
}

testColRealIPL <- function() {
    indir = system.file('/extdata/runPrd/', package='MPAC')
    pat = 'TCGA-CV-7100'
    
    out_alldt = colRealIPL(indir, sampleids=NULL)
    out_patdt = colRealIPL(indir, sampleids=c(pat))

    cmpdt = system.file('/extdata/colIPL/real.rds', package='MPAC') |> readRDS()
    testColIPL('testColRealIPLAll', out_alldt, cmpdt)
    testColIPL('testColRealIPLPat', out_patdt, cmpdt)
}

testColPermIPL <- function() {
    indir = system.file('/extdata/runPrd/', package='MPAC')
    n_perms = 3
    pat = 'TCGA-CV-7100'

    out_alldt = colPermIPL(indir, n_perms, sampleids=NULL)
    out_patdt = colPermIPL(indir, n_perms, sampleids=c(pat))

    cmpdt = system.file('/extdata/colIPL/perm.rds', package='MPAC') |> readRDS()
    testColIPL('testColRealIPLAll', out_alldt, cmpdt)
    testColIPL('testColRealIPLPat', out_patdt, cmpdt)
}

testColIPL <- function(tag, outdt, cmpdt) {
    test_that(tag, {
        expect_identical(outdt, cmpdt)
    })
}

main()

#' @title  Filter IPLs from real data by distribution from permuted data
#'
#' @param realdt  A data.table object containing entities and their IPLs from
#'                real data. It is the output from `colRealIPL()`.
#'
#' @param permdt  A data.table object containing permutation index, entities
#'                and their IPLs from permuted data. It is the output from
#'                `colPermIPL()`.
#'
#' @param threads  Number of threads to run in parallel. Default: 1
#'
#' @return A matrix of filtered IPLs with rows as entities and columns as
#'         samples. Entities with IPLs observed by chance are set to NA.
#'
#' @export
#'
#' @examples
#'
#' freal = system.file('extdata/fltByPerm/real.rds', package='MPAC')
#' fperm = system.file('extdata/fltByPerm/perm.rds', package='MPAC')
#' realdt = readRDS(freal)
#' permdt = readRDS(fperm)
#'
#' fltByPerm(realdt, permdt)
#'
#' @importFrom BiocParallel  SnowParam bplapply
#'
fltByPerm <- function(realdt, permdt, threads=1) {
    pats <- intersect(names(realdt), names(permdt)) |> setdiff('entity')

    bp <- getBPPARAM(threads)
    bplapply(pats, fltByPat, realdt, permdt, BPPARAM=bp) |> rbindlist() |>
    dcast(entity ~ pat, value.var='flt_real_ipl') |>
    as.matrix(rownames='entity')
}

#' @importFrom stats  mad median
fltByPat <- function(pat, in_realdt, in_permdt) {
    realdt <- in_realdt[, c('entity', pat), with=FALSE] |>
        setnames(pat, 'real_ipl')

    perm_mat <- in_permdt[, c('entity', 'iperm', pat), with=FALSE] |>
        dcast(iperm ~ entity, value.var=pat) |> as.matrix(rownames='iperm')

    median_vec <- apply(perm_mat, 2, median)
    mad_vec    <- apply(perm_mat, 2, mad)

    perm_median <- perm_mad <- is_real <- med_m_3mad <- med_p_3mad <- NULL
    . <- real_ipl <- flt_real_ipl <- entity <- NULL

    maddt <- data.table(
        entity      = names(mad_vec),
        perm_median = median_vec,
        perm_mad    = mad_vec ) |>
    merge(realdt, y=_, by='entity', all=TRUE) |>
    _[, `:=`(
        med_m_3mad = perm_median - 3 * perm_mad,
        med_p_3mad = perm_median + 3 * perm_mad )] |>
    _[, is_real := ifelse( (real_ipl < med_m_3mad) |
                            (real_ipl > med_p_3mad), TRUE, FALSE )]

    maddt[, flt_real_ipl := ifelse(is_real == TRUE, real_ipl, NA)] |>
    _[, pat := pat] |>
    _[, .(pat, entity, flt_real_ipl)]
}

main <- function() {
    testFltByPerm()
}

testFltByPerm <- function() {
    realdt = system.file('extdata/fltByPerm/real.rds', package='MPAC') |> 
             readRDS()
    permdt = system.file('extdata/fltByPerm/perm.rds', package='MPAC') |>
             readRDS()

    outmat = fltByPerm(realdt, permdt)

    cmpmat = system.file('extdata/fltByPerm/flt_real.rds', package='MPAC') |>
             readRDS()

    test_that('testFltByPerm', {
        expect_identical(outmat, cmpmat)
    })
}

main()

#' @title  Plot a heatmap of IPLs on proteins from consensus pathway submodules
#'
#' @param grphl  A list of igraph objects representing consensus pathway
#'               submodules. It is the output from `conMtf()`.
#'
#' @param proteins  A vector of proteins, of which IPLs to plot. Default: all
#'                  proteins that in both `grphl` and `fltmat`.
#'
#' @inheritParams pltNeiStt
#' @inheritParams pltOvrHm
#'
#' @return  A heatmap of IPLs o proteins from consensus pathway submodules
#'
#' @examples
#'
#' fltmat <- system.file('extdata/pltSttKM/ipl.rds',package='MPAC') |> readRDS()
#' cldt <- system.file('extdata/pltMtfPrtIPL/cl.rds',package='MPAC')|> readRDS()
#' grphl <- system.file('extdata/pltMtfPrtIPL/grphl.rds',package='MPAC') |>
#'          readRDS()
#'
#' pltMtfPrtIPL(fltmat, cldt, grphl, proteins=c('CD247', 'FASLG'))
#'
#' @export
#'
#' @import igraph
#' @import ComplexHeatmap
#' @importFrom circlize colorRamp2
#'
pltMtfPrtIPL <- function(fltmat, cldt, grphl, proteins=NULL){
    protdt = findJntProts(fltmat, grphl, proteins)

    nreps = NULL
    setnames(cldt, 1, 'nreps')
    srt_cldt = cldt[order(-nreps)][1, ] |>
               melt(id='nreps', variable='brc', value='icl')

    jnt_brcs = intersect(srt_cldt$brc, colnames(fltmat))
    brc = NULL
    sel_srt_cldt = srt_cldt[ brc %in% jnt_brcs ]
    pltmat = fltmat[protdt$prot, sel_srt_cldt$brc, drop=FALSE]

    hmMtfIplByCl(pltmat, protdt, sel_srt_cldt)
}

hmMtfIplByCl <- function(pltmat, protdt, cldt) {
    icl_lab = icl = NULL
    cldt[, icl_lab := paste0('c', icl, "\nn=", .N), by=icl]

    FONT_SIZE = 9
    CYAN    = '#56B4E9'
    MAGENTA = '#CC79A7'
    Heatmap( pltmat,
        col = circlize::colorRamp2(c(-0.5, 0, 0.5), c(CYAN, 'gray95', MAGENTA)),
        na_col = 'black',
        border            = 'black',
        rect_gp           = gpar(col='white'),
        column_split      = cldt$icl_lab,
        row_split         = protdt$mtf,
        show_row_names    = TRUE,
        show_column_names = FALSE,
        column_names_side = 'top',
        row_names_gp      = gpar(fontsize=FONT_SIZE),
        column_title_gp   = gpar(fontsize=FONT_SIZE),
        row_title_rot     = 0,
        row_title_gp      = gpar(fontsize=FONT_SIZE),
        show_row_dend     = FALSE,
        show_column_dend  = FALSE,
        cluster_column_slices = FALSE,
        cluster_row_slices    = FALSE,
        cluster_columns   = TRUE,
        cluster_rows      = FALSE,
        use_raster        = FALSE,
        heatmap_legend_param = list(
            title       = 'IPL',
            title_gp    = gpar(fontsize=FONT_SIZE),
            labels_gp   = gpar(fontsize=FONT_SIZE),
            grid_height = grid::unit(3, 'mm'),
            grid_width  = grid::unit(3, 'mm'),
            border      = 'black',
            title_position = 'lefttop',
            direction   = 'horizontal'
        )
    ) |> draw(
        heatmap_legend_side = 'bottom',
        column_title = paste0(ncol(pltmat), ' samples'),
        column_title_gp = gpar(fontsize=FONT_SIZE),
        background = 'transparent'
    )
}

findJntProts <- function(fltmat, grphl, proteins) {
    if ( is.null(proteins) ) {
        proteins = rownames(fltmat)
    }

    grph_nodes = do.call(igraph::union, grphl) |> V() |> _[]$name
    prots = rownames(fltmat) |> intersect(grph_nodes) |> intersect(proteins)
    lapply(seq_along(grphl), function(isub) {
        data.table(mtf = paste0('submodule ', isub),
                   prot = intersect(prots, V(grphl[[isub]])$name))
    }) |> rbindlist()
}

main <- function() {
    testPltMtfPrtIPL()
}

testPltMtfPrtIPL <- function() {
    fltmat = system.file('extdata/pltSttKM/ipl.rds',package='MPAC') |> readRDS()
    cldt = system.file('extdata/pltMtfPrtIPL/cl.rds',package='MPAC')|> readRDS()
    grphl = system.file('extdata/pltMtfPrtIPL/grphl.rds',package='MPAC') |>
            readRDS()

    test_that('testPltMtfPrtIPL', {
        pltMtfPrtIPL(fltmat, cldt, grphl, proteins=c('CD247', 'FASLG')) |>
        print() |>
        expect_no_error()
    })

    fpdf = 'Rplots.pdf'
    if ( file.exists(fpdf) ) file.remove(fpdf)
}

main()

#' @title  Run PARADIGM on multi-omic data
#'
#' @param real_se  A SummarizedExperiment object of PARADIGM CNA and RNA states.
#'                 It is the same matrix as the output from `ppRealInp()`.
#'
#' @param fpth  Name of a pathway file for PARADIGM.
#'
#' @param outdir  Output folder to save all results.
#'
#' @param PARADIGM_bin  PARADIGM binary, which can be downloaded from
#' https://github.com/sng87/paradigm-scripts/tree/master/public/exe. Note that
#' the binary is only available for Linux or MacOS. Default: NULL
#'
#' @param nohup_bin  nohup binary, which is used for long running PARADIGM jobs.
#'                   Default: NULL
#'
#' @param sampleids  A vector of sample IDs to run PARADIGM on. If not provided,
#'                   all the samples that exist in both copy-number alteration
#'                   and RNA files will be ran. Default: NULL
#'
#' @param file_tag  A string of output file name tag. Default: NULL
#'
#' @inheritParams ppRnaInp
#'
#' @usage runPrd(real_se, fpth, outdir, PARADIGM_bin=NULL, nohup_bin=NULL,
#'     sampleids=NULL, file_tag=NULL, threads=1)
#'
#' @return None
#'
#' @export
#'
#' @examples
#'
#' freal = system.file('extdata/TcgaInp/inp_real.rds', package='MPAC')
#' real_se  = readRDS(freal)
#'
#' fpth = system.file('extdata/Pth/tiny_pth.txt', package='MPAC')
#' outdir = tempdir()
#' paradigm_bin = '/path/to/PARADIGM'  ## change to binary location
#'
#' # depends on external PARADIGM binary
#' runPrd(real_se, fpth, outdir, paradigm_bin, sampleids=c('TCGA-CV-7100'))
#'
#' @importFrom BiocParallel bplapply
#' @importFrom SummarizedExperiment assays
#'
runPrd <- function(real_se, fpth, outdir, PARADIGM_bin=NULL,
    nohup_bin=NULL, sampleids=NULL, file_tag=NULL, threads=1) {

    if ( is.null(sampleids) ) {
        sampleids <- colnames(real_se)
    }

    prots <- rownames(real_se) |> sort()
    cn_state_mat  <- assays(real_se)$CN_state  |>
        _[prots, sampleids, drop=FALSE] |> t()
    rna_state_mat <- assays(real_se)$RNA_state |>
        _[prots, sampleids, drop=FALSE] |> t()

    # jntl <- findJntPatProt(cn_state_mat, rna_state_mat, fpth, sampleids)
    if ( ! file.exists(outdir)) dir.create(outdir, recursive=TRUE)

    pats <- rownames(cn_state_mat)
    dummy <- bplapply(pats, runPrdByPat, cn_state_mat, rna_state_mat, fpth,
        outdir, PARADIGM_bin, nohup_bin, file_tag, BPPARAM=getBPPARAM(threads))
}

#' @title  Run PARADIGM on permuted data
#'
#' @param perml  A list of SummarizedExperiment objects of permuted CNA and
#'               RNA states generated by `ppPermInp()`.
#'
#' @inheritParams runPrd
#'
#' @usage runPermPrd(perml, fpth, outdir,
#'     PARADIGM_bin=NULL, nohup_bin=NULL, sampleids=NULL, threads=1)
#'
#' @return None
#'
#' @export
#'
#' @examples
#'
#' fperm = system.file('extdata/TcgaInp/inp_perm.rds', package='MPAC')
#' perml = readRDS(fperm)
#' fpth = system.file('extdata/Pth/tiny_pth.txt', package='MPAC')
#' outdir = tempdir()
#' paradigm_bin = '/path/to/PARADIGM'  ## change to binary location
#' pat = 'TCGA-CV-7100'
#'
#' # depends on external PARADIGM binary, do not run
#' runPermPrd(perml, fpth, outdir, paradigm_bin, sampleids=c(pat))
#'
#' @importFrom BiocParallel bplapply
#' @importFrom S4Vectors metadata
#'
runPermPrd <- function(perml, fpth, outdir, PARADIGM_bin=NULL, nohup_bin=NULL,
    sampleids=NULL, threads=1) {

    dummy <- lapply(perml, function(perm_se) {
        iperm_tag <- paste0('p', metadata(perm_se)$i)
        iperm_outdir <- paste0(outdir, '/', iperm_tag, '/')
        if ( is.null(sampleids) ) {
            sampleids <- colnames(perm_se)
        }
        runPrd(perm_se, fpth, iperm_outdir, PARADIGM_bin, nohup_bin, sampleids,
            threads, file_tag=metadata(perm_se)$i)
    })
}

runPrdByPat <- function(pat, cnmat, rnamat, fpth, outdir, PARADIGM_bin,
    nohup_bin, file_tag=NULL) {

    outl <- ppRunPrd(pat, cnmat, rnamat, outdir, file_tag)
    fcfg <- outl$fcfg
    fcpt <- outl$fcpt
    fipl <- outl$fipl
    out_prefix <- outl$out_prefix

    runPARADIGM(fcfg, fcpt, fipl, out_prefix, fpth, PARADIGM_bin, nohup_bin)
}

#' @title  Prepare required files to run PARADIGM
#'
#' @param pat  Sample ID
#'
#' @param cnmat  CN matrix
#'
#' @param rnamat  RNA matrix
#'
#' @inheritParams runPrd
#'
#' @usage ppRunPrd(pat, cnmat, rnamat, outdir, file_tag=NULL)
#'
#' @return None
#'
#' @export
#'
ppRunPrd <- function(pat, cnmat, rnamat, outdir, file_tag=NULL) {
    fone_samp <- tag <- NULL
    if ( is.null(file_tag) ) {
        file_tag <- pat
    } else {
        file_tag <- paste0(pat, '_', file_tag)
    }
    out_prefix <- paste0(outdir, '/', file_tag, '_')
    omicdt <- data.table( rbind(
        c( 'genome', 'inp_focal'       ),
        c( 'mRNA',   'inp_log10fpkmP1' )
    )) |> setnames( c('type', 'tag') ) |>
    _[, fone_samp := paste0(out_prefix, tag, '.tsv')]

    type2mat <- list(
        'genome' = cnmat[ pat, , drop=FALSE],
        'mRNA'   = rnamat[pat, , drop=FALSE]
    )

    fcfg <- paste0(out_prefix, 'cfg.txt')
    fcpt <- paste0(out_prefix, 'cpt.txt')
    fipl <- paste0(out_prefix, 'ipl.txt')

    Map(prepOmic, omicdt$type, omicdt$fone_samp, MoreArgs=list(type2mat))

    prepConfig(fcfg, omicdt)

    list(fcfg=fcfg, fcpt=fcpt, fipl=fipl, out_prefix=out_prefix)
}

prepOmic <- function(type, fone_samp, type2mat) {
    type2mat[[type]] |>
    as.data.table(keep.rownames=TRUE) |> setnames('rn', 'pat') |>
    fwrite(fone_samp, quote=FALSE, sep="\t")
}

runPARADIGM <- function(fcfg, fcpt, fipl, out_prefix, fpth, PARADIGM_bin,
    nohup_bin){

    args <- c(
        '-p', fpth,
        '-c', fcfg,
        '-b', dirname(out_prefix) |> paste0('/'),
        '-e', fcpt,
        '-o', fipl,
        '-m 1'  ## max_mem_in_G
    )

    fout <- paste0(out_prefix, 'run.out')
    ferr <- paste0(out_prefix, 'run.err')

    if ( is.null(PARADIGM_bin) ) {
        paste0("Please provide PARADIGM binary. See vignette for details\n") |>
        cat()
    } else if ( ! file.exists(PARADIGM_bin) ) {
        paste0('PARADIGM binary not found:', PARADIGM_bin, "\n") |> cat()
    } else {
        ## cat(PARADIGM_bin, args, "\n")
        if ( is.null(nohup_bin) ) {
            system2(PARADIGM_bin, args=args, stdout=fout, stderr=ferr)
        } else {
            args <- c(PARADIGM_bin, args)
            system2(nohup_bin, args=args, stdout=fout, stderr=ferr)
        }
    }
}

prepConfig <- function(fcfg, omicdt) {
    s_obss <- c()
    s_evis <- c()
    options(scipen=10) ## for cnv_cutoff == 0.0001
    for ( i in seq_len(nrow(omicdt)) ) {
        fname_one_samp <- basename(omicdt[i]$fone_samp)
        s_obss <- c(s_obss, paste0(fname_one_samp, '=-obs>'))
        s_evis <- c(s_evis, paste0('evidence [suffix=', fname_one_samp,
            ',node=', omicdt[i]$type, ',disc=-0.5;0.5',
            ',epsilon=0.001,epsilon0=0.2]'))
    }

    paste0(
        "pathway [max_in_degree=5]\n",
        "inference [method=BP,updates=SEQFIX,tol=1e-9,maxiter=10000,",
            "logdomain=0]\n",
        'em_step [', paste(s_obss, collapse=','), "]\n",
        "em [max_iters=10000,log_z_tol=1e-10]\n",
        paste(s_evis, collapse="\n") ) |>
    write(file=fcfg)
}

main <- function() {
    # paradigm_bin = dlParadigmBin()

    testRunRealPrd()

    testRunPermPrd()
}

testRunPermPrd <- function() {
    fpermll = system.file('extdata/TcgaInp/inp_perm.rds', package='MPAC')
    fpth    = system.file('extdata/Pth/tiny_pth.txt',     package='MPAC')

    permll = readRDS(fpermll)
    pat = 'TCGA-CV-7100'
    outdir = tempdir()

    runPermPrd(permll, fpth, outdir, PARADIGM_bin=NULL, sampleids=c(pat))

    lapply(permll, testRunPermPrdByIPerm, pat, outdir)
}

testRunPermPrdByIPerm <- function(perm_se, pat, outdir) {
    iperm = S4Vectors::metadata(perm_se)$i
    out_prefix = paste0(outdir, '/p', iperm, '/', pat, '_', iperm, '_')
    cmp_relpre = paste0('extdata/runPrd/p', iperm, '/', pat, '_', iperm, '_')
    testRunPrdByPrefix('testRunPermPrd', out_prefix, cmp_relpre)
}

testRunRealPrd <- function() {
    freal = system.file('extdata/TcgaInp/inp_real.rds', package='MPAC')
    fpth  = system.file('extdata/Pth/tiny_pth.txt',     package='MPAC')

    real_se = readRDS(freal)
    pat = 'TCGA-CV-7100'

    outdir = tempdir()
    runPrd(real_se, fpth, outdir, PARADIGM_bin=NULL, sampleids=c(pat))

    out_prefix = paste0(outdir, '/', pat, '_')
    cmp_relpre = paste0('extdata/runPrd/', pat, '_')

    testRunPrdByPrefix('testRunRealPrd', out_prefix, cmp_relpre)
}

testRunPrdByPrefix <- function(testname, out_prefix, cmp_relpre) {
    # suffixes = c('cfg.txt', 'inp_focal.tsv', 'inp_log10fpkmP1.tsv', 'cpt.txt',
    #     'ipl.txt')
    suffixes = c('cfg.txt', 'inp_focal.tsv', 'inp_log10fpkmP1.tsv')
    lapply(suffixes, testFileContent, testname, out_prefix, cmp_relpre)
}

testFileContent <- function(suffix, testname, out_prefix, cmp_relpre) {
    fout = paste0(out_prefix, suffix)
    fcmp = paste0(cmp_relpre, suffix) %>% system.file(package='MPAC')
    outlines = readLines(fout)
    cmplines = readLines(fcmp)

    test_that(paste0(testname, ':', suffix), {
        expect_identical(outlines, cmplines)
    })
}

main()

#' @title  Calculate over-representation of gene sets in each sample by genes
#'         from sample's largest sub-pathway
#'
#' @param  subntwlist  A list of igraph objects represented the largest 
#'                     sub-pathway for each sample. It is the output of 
#'                     `subNtw()`.
#'
#' @param  omic_genes  A vector of gene symbols to narrow down 
#'                     over-representation calculation to only those with input
#'                     genomic data. If not provided, all genes in the GMT file
#'                     will be considered. Default: NULL.
#'
#' @inheritParams subNtw
#'
#' @return  A matrix containing over-representation adjusted P with rows as 
#'          gene set names and columns as sample IDs.
#'
#' @export
#'
#' @examples
#'
#' fsubntwl  = system.file('extdata/subNtw/subntwl.rds',    package='MPAC')
#' fgmt      = system.file('extdata/ovrGMT/fake.gmt',       package='MPAC')
#' fomic_gns = system.file('extdata/TcgaInp/inp_focal.rds', package='MPAC')
#' subntwl  = readRDS(fsubntwl)
#' omic_gns = rownames(readRDS(fomic_gns))
#'
#' ovrGMT(subntwl, fgmt, omic_gns)
#'
#'
#' @import igraph
#' @importFrom fgsea gmtPathways
#'
ovrGMT <- function(subntwlist, fgmt, omic_genes=NULL, threads=1) {
    gmtl <- gmtPathways(fgmt)
    gmt_gns <- do.call(c, gmtl) |> unique()

    urn_balls <- NULL
    if ( is.null(omic_genes) ) {
        urn_balls <- gmt_gns
    } else {
        urn_balls <- intersect(gmt_gns, omic_genes)
    }

    getBPPARAM(threads) |> 
    bplapply(names(subntwlist), getOvrSubNtwByPat, gmtl, urn_balls, subntwlist,
        BPPARAM=_) |>
    rbindlist() |> dcast(goname ~ pat, value.var='fisher_padj') |>
    as.matrix(rownames='goname')
}

#' @importFrom stats p.adjust
getOvrSubNtwByPat <- function(pat, gmtl, urn_balls, subntwlist) {
    fisher_padj <- ipl <- name <- fisher_pval <- NULL

    subgrph <- subntwlist[[pat]]
    sel_ents <- as_data_frame(subgrph, what='vertices') |> data.table() |>
        _[ abs(ipl) > 0 ]$name

    urn_white_balls <- intersect(sel_ents, urn_balls)

    lapply(names(gmtl), defOvrByGmt, gmtl, urn_balls, urn_white_balls) |>
    rbindlist() |> _[order(fisher_pval)] |>
    _[, fisher_padj := p.adjust(fisher_pval, method='BH')] |>
    _[, pat := pat]
}

#' @importFrom stats fisher.test
defOvrByGmt <- function(goname, gmtl, urn_balls, urn_white_balls) {
    p.value <- NULL

    gmt_gns <- gmtl[[goname]]
    drawn_balls         <- intersect(gmt_gns, urn_balls)
    n_drawn_white_balls <- intersect(drawn_balls, urn_white_balls) |> length()
    n_drawn_black_balls <- length(drawn_balls) - n_drawn_white_balls
    n_not_drawn_balls   <- length(urn_balls) - length(drawn_balls)
    n_not_drawn_white_balls <- length(urn_white_balls) - n_drawn_white_balls
    n_not_drawn_black_balls <- n_not_drawn_balls - n_not_drawn_white_balls

    if ( n_drawn_white_balls > 0 ) {
        fisher_pval <- matrix(c(n_drawn_white_balls, n_not_drawn_white_balls,
            n_drawn_black_balls, n_not_drawn_black_balls),
            nrow=2, byrow=TRUE) |> fisher.test() |> _$p.value
    } else {
        fisher_pval <- NA
    }

    list( 
        goname = goname,
        n_drawn_white_gns     = n_drawn_white_balls,
        n_drawn_black_gns     = n_drawn_black_balls,
        n_not_drawn_white_gns = n_not_drawn_white_balls,
        n_not_drawn_black_gns = n_not_drawn_black_balls,
        fisher_pval = fisher_pval 
    )
}

main <- function() {
    testOvrGMT()
}

testOvrGMT <- function() {
    fntw   = system.file('extdata/subNtw/subntwl.rds',    package='MPAC')
    fgmt   = system.file('extdata/ovrGMT/fake.gmt',       package='MPAC')
    ffocal = system.file('extdata/TcgaInp/inp_focal.rds', package='MPAC')
    fcmp   = system.file('extdata/ovrGMT/ovr.rds',        package='MPAC')

    subntwl = readRDS(fntw) |> lapply(function(s) upgrade_graph(s))
    omic_gns = readRDS(ffocal) |> rownames()

    outmat = ovrGMT(subntwl, fgmt, omic_gns)
    cmpmat = readRDS(fcmp)

    test_that('testOvrGMT', {
        expect_equal(outmat, cmpmat)
    })
}

main()

#' @title  Cluster samples by pathway over-representation
#'
#' @param  ovrmat  A matrix of gene set over-representation adjusted p-values
#'                 with rows as gene sets and columns as samples. It is the
#'                 output from `ovrGMT()`.
#'
#' @param n_neighbors  Number of neighbors for clustering. A larger number is
#'                     recommended if the size of samples is large. Default: 10.
#'
#' @param n_random_runs  Number of random runs. Due to randomness introduced
#'                       to the Louvain algorithm in R igraph 1.3.0
#'                       (https://github.com/igraph/rigraph/issues/539), a large
#'                       number of runs are recommended to evaluate randomness
#'                       in the clustering results. Default: 200, which shall be
#'                       safe for sample size < 50. Please increase it
#'                       accordingly for a larger sample size.
#'
#' @inheritParams ppRnaInp
#'
#' @return  A data table with each row representing one clustering result, and
#'          the first column denotes the number of occurrences of a clustering
#'          result and the rest of columns indicating each sample's cluster
#'          index. Rows are ordered by the number of occurrences from high to
#'          low.
#'
#' @examples
#'
#' fovr = system.file('extdata/clSamp/ovrmat.rds', package='MPAC')
#' ovrmat = readRDS(fovr)
#'
#' clSamp(ovrmat)
#'
#' @export
#'
#' @import SingleCellExperiment
#' @importFrom scran modelGeneVar getTopHVGs denoisePCA clusterCells
#' @importFrom BiocSingular RandomParam
#' @importFrom bluster NNGraphParam
#' @importFrom SummarizedExperiment rowData<-
#' @importFrom S4Vectors DataFrame
#' @importFrom BiocParallel bplapply
#'
clSamp <- function(ovrmat, n_neighbors=10, n_random_runs=200, threads=1) {
    n_top_hvgs <- 100

    ovrmat <- ovrmat[, sort(colnames(ovrmat))]
    ovrmat[is.na(ovrmat)] <- 1.0
    sce <- SingleCellExperiment(
        list(logcounts = abs(log10(ovrmat))),
        colData = DataFrame(samp=colnames(ovrmat)),
        rowData = DataFrame(goname=rownames(ovrmat))
    )

    bp <- getBPPARAM(threads)
    dec <- modelGeneVar(sce, BPPARAM=bp)
    top_hvgs <- getTopHVGs(dec, n=n_top_hvgs)
    rowData(sce)$is_top_hvgs <- (rownames(sce) %in% top_hvgs)

    sce <- denoisePCA(sce, subset.row=top_hvgs, technical=dec,
        BSPARAM=RandomParam(), BPPARAM=bp)

    ngp <- NNGraphParam(k=n_neighbors, type='jaccard', cluster.fun='louvain')

    pats <- colnames(sce)

    cldt <- bplapply(seq_len(n_random_runs), function(irep) {
        clusterCells(sce, use.dimred='PCA', BLUSPARAM=ngp) |> as.integer() |>
        data.table(irep=irep, pat=pats, icl=_)
    }, BPPARAM=bp) |> rbindlist() |>
    dcast(irep ~ pat, value.var='icl')

    nreps <- NULL
    cldt[, list(nreps = .N), by=pats] |>
    _[, c('nreps', pats), with=FALSE] |> _[order(-nreps)]
}

main <- function() {
    testClSamp()
}

testClSamp <- function() {
    fovr = system.file('extdata/clSamp/ovrmat.rds', package='MPAC')
    fcmp = system.file('extdata/clSamp/clmat.rds',  package='MPAC')

    ovrmat = readRDS(fovr)
    outdt = clSamp(ovrmat, n_neighbors=10, n_random_runs=5) %>%
            suppressWarnings()
    
    ## take the most frequent one
    freqmat = outdt[order(-nreps)] %>% as.matrix(rownames='nreps') %>%
        .[1, ] %>% t() %>% t()
    colnames(freqmat) = 'icl'
    cmpmat = readRDS(fcmp)

    test_that('testClSamp', {
        expect_identical(freqmat, cmpmat)
    })
}

main()

FONT_SIZE <- 11
MAGENTA <- '#CC79A7'
CYAN    <- '#56B4E9'

DASHDT <- data.table( rbind(
    c( '-a|',        'protein repression'         ),
    c( '-t|',        'transcriptional repression' ),
    c( 'component>', 'is a component of'          ),
    c( 'member>',    'is a member of'             ),
    c( '-a>',        'protein activation'         ),
    c( '-t>',        'transcriptional activation' ),
    c( '-omic>',     'omic'                       )
)) |> setnames( c('title', 'desc') )

OMIC_NODEDT <- data.table( rbind(
    c( 'CNA',     'omic' ),
    c( 'RNA-seq', 'omic' )
)) |> setnames( c('entity', 'type') )

OMIC_EDGEDT <- data.table( rbind(
    c( 'CNA',     '-omic>' ),
    c( 'RNA-seq', '-omic>' )
)) |> setnames( c('from', 'title') )

#' @title  Plot a heatmap of over-represented gene sets for clustered samples
#'
#' @param  ovrmat   A matrix containing over-representation adjusted P with
#'                  rows as gene set names and columns as sample IDs. It is the
#'                  output of the `ovrGMT()` function.
#'
#' @param  cldt     A data table with each row representing one clustering
#'                  result, and the first column denotes the number of
#'                  occurrences of a clustering result and the rest of columns
#'                  indicating each sample's cluster index. It is the output
#'                  of the `clSamp()` function. Only the most frequent
#'                  clustering result will be used to plot.
#'
#' @param  min_frc  A minimum fraction of samples in a cluster that have a gene
#'                  set significantly over-represented (adjusted P < 0.05).
#'                  This is used to select gene sets to plot. Default: 0.8
#'
#' @return  A heatmap with rows as over-represented gene sets and columns as
#'          samples splited by clusters.
#'
#' @examples
#'
#' ovrmat <- system.file('extdata/pltOvrHm/ovr.rds',package='MPAC') |> readRDS()
#' cldt   <- system.file('extdata/pltOvrHm/cl.rds', package='MPAC') |> readRDS()
#'
#' pltOvrHm(ovrmat, cldt)
#'
#' @export
#'
pltOvrHm <- function(ovrmat, cldt, min_frc=0.8) {
    outl <- getSignifOvrOnCl(ovrmat, cldt, min_frc)

    makeOvrHm(log10(outl$padjmat), outl$srt_cldt, min_frc)
}

#' @title  Get significantly over-represented gene sets for clustered samples
#'
#' @inheritParams pltOvrHm
#'
#' @return  A list of a matrix and a data.table object. The matrix has rows as
#'          over-represented gene sets, columns as samples, and each cell
#'          stores an adjusted P for over-representation. The data.table has
#'          the clustering informations with samples in the same order as the
#'          matrix's column.
#'
#' @examples
#'
#' ovrmat <- system.file('extdata/pltOvrHm/ovr.rds',package='MPAC') |> readRDS()
#' cldt   <- system.file('extdata/pltOvrHm/cl.rds', package='MPAC') |> readRDS()
#'
#' getSignifOvrOnCl(ovrmat, cldt)
#'
#' @export
#'
getSignifOvrOnCl <- function(ovrmat, cldt, min_frc=0.8) {
    nreps <- nsamps <- icl <- padj <- is_signif <- . <- goname <- frc <- NULL

    setnames(cldt, 1, 'nreps')
    srt_cldt <- cldt[order(-nreps)][1, ] |>
        melt(id='nreps', variable='brc', value='icl') |>
        _[, nsamps := .N, by=icl] |> _[, nreps := NULL]

    gonames <- as.data.table(ovrmat, keep.rownames='goname') |>
        melt(id='goname', variable='brc', value='padj') |>
        _[, padj := ifelse(is.na(padj), 1.0, padj)] |>
        _[, is_signif := ifelse(padj < 0.05, TRUE, FALSE)] |>
        merge(srt_cldt, by='brc', all.x=TRUE) |>
        _[, .(frc = sum(is_signif)/nsamps), by=.(goname, icl)] |>
        _[ frc >= min_frc ]$goname |> unique()

    list(padjmat = ovrmat[gonames, srt_cldt$brc], srt_cldt = srt_cldt)
}

#' @import ComplexHeatmap
#' @importFrom grid  gpar unit
#' @importFrom stringr  str_wrap
#' @importFrom scales   percent
#' @importFrom circlize colorRamp2
#' @importFrom viridis  cividis
#'
makeOvrHm <- function(pltmat, cldt, min_frc) {
    icl_lab <- icl <- nsamps <- NULL

    OVR_CLRS <- colorRamp2(seq(-4, 0, 0.1), rev(cividis(41)))
    FONT_SIZE <- 9
    row_title <- paste0( nrow(pltmat),
        ' gene sets significantly over-represented in >= ', percent(min_frc),
        ' samples in a group') |> str_wrap(width=45)
    cldt[, icl_lab := paste0('c', icl, "\nn=", nsamps)]

    rownames(pltmat) <- gsub('_', ' ', rownames(pltmat))
    pltmat <- pltmat[, as.character(cldt$brc)]

    Heatmap( pltmat,
        col = OVR_CLRS,
        na_col = 'black',
        height = (nrow(pltmat) + 1) * unit(3, 'mm'),
        border            = 'black',
        column_split      = cldt$icl_lab,
        column_title_gp   = gpar(fontsize=FONT_SIZE),
        cluster_column_slices = FALSE,
        show_row_names    = TRUE,
        row_names_gp      = gpar(fontsize=7),
        show_column_names = FALSE,
        column_names_side = 'top',
        column_names_gp   = gpar(fontsize=FONT_SIZE),
        show_row_dend     = FALSE,
        show_column_dend  = FALSE,
        row_title         = row_title,
        row_title_gp      = gpar(fontsize=FONT_SIZE),
        cluster_columns   = TRUE,
        cluster_rows      = TRUE,
        use_raster        = ifelse( (nrow(pltmat) > 20) | (ncol(pltmat) > 20),
            TRUE, FALSE),
        raster_quality    = 5,
        heatmap_legend_param = list(
            title       = expression(paste('log'[10], '(adjusted p)')),
            title_gp    = gpar(fontsize=FONT_SIZE),
            labels_gp   = gpar(fontsize=FONT_SIZE),
            legend_width = unit(0.8, 'inch'),
            title_position = 'lefttop',
            direction   = 'horizontal'
        )
    ) |> draw(
        background      = 'transparent',
        heatmap_legend_side = 'bottom',
        column_title    = paste0(ncol(pltmat), ' samples in total'),
        column_title_gp = gpar(fontsize=FONT_SIZE),
        padding = unit(c(2, 10, 2, 25), 'mm')
    )
}

main <- function() {
    testPltOvrHm()
}

testPltOvrHm <- function() {
    fovr = system.file('extdata/pltOvrHm/ovr.rds', package='MPAC')
    fcl  = system.file('extdata/pltOvrHm/cl.rds',   package='MPAC')

    ovrmat = readRDS(fovr)
    cldt   = readRDS(fcl)

    test_that('testPltOvrHm', {
        pltOvrHm(ovrmat, cldt) |>
        expect_no_error()
    })

    fpdf = 'Rplots.pdf'
    if ( file.exists(fpdf) ) file.remove(fpdf)
}

main()

#' @title  Subset pathways by IPL results
#'
#' @param fltmat  A matrix contains filterd IPL with rows
#'                as 'entity' and column as samples. This is the output from
#'                `fltByPerm()`. Entity with NA in all columns will be ignored.
#'
#' @param fgmt  A gene set GMT file. This will be the same file used for the
#'              gene set over-representation calculation in the next step. It
#'              is used here to ensure output sub-pathway contains a minimum
#'              number of genes from to-be-used gene sets.
#'
#' @param min_n_gmt_gns  Minimum number of genes from the GMT file in the output
#'                       sub-pathway. Default: 2.
#'
#' @inheritParams ppRnaInp
#' @inheritParams runPrd
#'
#' @return  A list of igraph objects representing the largest sub-pathway
#'          for each sample.
#'
#' @export
#'
#' @examples
#'
#' fflt = system.file('extdata/fltByPerm/flt_real.rds', package='MPAC')
#' fltmat = readRDS(fflt)
#' fpth = system.file('extdata/Pth/tiny_pth.txt',       package='MPAC')
#' fgmt = system.file('extdata/ovrGMT/fake.gmt',        package='MPAC')
#'
#' subNtw(fltmat, fpth, fgmt, min_n_gmt_gns=1)
#'
#'
#' @importFrom fgsea gmtPathways
#'
subNtw <- function(fltmat, fpth, fgmt, min_n_gmt_gns=2, threads=1) {
    is_all_na <- apply(fltmat, 1, function(x) all(is.na(x)))
    fltmat <- fltmat[ ! is_all_na, ]
    pthl <- getNodeEdge(fpth)
    nodedt <- pthl$nodedt
    edgedt <- pthl$edgedt

    gmt_gns <- gmtPathways(fgmt) |> do.call(c, args=_) |> unique()
    sampleids <- colnames(fltmat)

    outl <- getBPPARAM(threads) |>
        bplapply(sampleids, getSubNtwByPat, nodedt, edgedt, fltmat, gmt_gns,
        min_n_gmt_gns, BPPARAM=_)

    names(outl) <- sampleids
    return(outl)
}

#' @import igraph
#'
getSubNtwByPat <- function(pat, in_nodedt, in_edgedt, fltmat, gmt_gns,
    min_n_gmt_gns) {

    nents <- from <- to <- entity <- isub <- NULL

    ipls <- fltmat[, pat]
    ents <- ipls[ ! is.na(ipls) ] |> names()

    ipldt <- fltmat[, pat, drop=FALSE] |>
        as.data.table(keep.rownames='entity') |> setnames(pat, 'ipl')
    edgedt <- in_edgedt[(from %in% ents) & (to %in% ents)]
    nodedt <- in_nodedt[(entity %in% edgedt$from) | (entity %in% edgedt$to)] |>
        merge(ipldt, by='entity', all.x=TRUE)

    subl <- graph_from_data_frame(d=edgedt, directed=TRUE, vertices=nodedt) |>
        decompose() |>
        Filter(function(g) {
            n_gmt_gns <- intersect(V(g)$name, gmt_gns) |> length()
            return(n_gmt_gns >= min_n_gmt_gns)
        }, x=_)

    ndt <- length(subl) |> seq_len() |> lapply(function(isub)
        list(isub=isub, nents=length(V(subl[[isub]])$name))) |> rbindlist()

    subntw <- subl[[ ndt[ nents == max(nents)]$isub ]]
    return(subntw)
}

require(data.table)
suppressMessages(require(igraph))

main <- function() {
    testSubNtw()
}

testSubNtw <- function() {
    fflt = system.file('extdata/fltByPerm/flt_real.rds', package='MPAC')
    fpth = system.file('extdata/Pth/tiny_pth.txt',       package='MPAC')
    fgmt = system.file('extdata/ovrGMT/fake.gmt',        package='MPAC')

    fltdt = readRDS(fflt)
    outl = subNtw(fltdt, fpth, fgmt, min_n_gmt_gns=1)
    cmpl = system.file('extdata/subNtw/subntwl.rds', package='MPAC') |>
           readRDS()

    lapply(names(outl), cmpSubNtwByPat, outl, cmpl)
}

cmpSubNtwByPat <- function(pat, outl, cmpl) {
    out = outl[[pat]]
    cmp = cmpl[[pat]] |> upgrade_graph()

    test_that('testSubNtw: vertex', {
        expect_identical( sort(V(out)$name), sort(V(cmp)$name) )
    })

    test_that('testSubNtw: edge', {
        expect_identical(
            as_edgelist(out) |> as.data.table() |> _[order(V1, V2)],
            as_edgelist(cmp) |> as.data.table() |> _[order(V1, V2)]
        )
    })
}

main()

## ----setup, include = FALSE--------------------------
knitr::opts_chunk$set( cache = TRUE, collapse = TRUE, echo = TRUE,
    message = FALSE, warning = FALSE, fig.align = 'center',
    dev.args=list(fix_text_size=FALSE ), comment = "#>")

options( htmltools.dir.version = FALSE, formatR.indent = 2,
    width = 55, digits = 4, warnPartialMatchAttr = FALSE,
    warnPartialMatchDollar = FALSE, tinytex.verbose=TRUE)

require(data.table)

require(MPAC)

## ----WorkflowJPG, fig.cap='MPAC workflow', out.width='800px', echo=FALSE----
knitr::include_graphics('workflow.jpg')

## ----pseudocode, eval=FALSE--------------------------
# FOR each sample:
#     ## Step 1
#     prepare CNA states from real data
# 
#     prepare RNA states from real data
# 
#     ## Step 2
#     compute IPLs by PARADIGM using CNA and RNA states from real data as well as
#     the input pathway definition
# 
#     ## Step 3
#     REPEAT 100 times:
#         randomly permute CNA and RNA states between genes
# 
#         compute IPLs by PARADIGM using CNA and RNA states from permuted data as
#         well as the input pathway definition
# 
#     ## Step 4
#     FOR each pathway entity:
#         filter entity’s IPL from real data by its 100 IPLs from permuted data
# 
#     ## Step 5
#     based on the input pathway definition, select the largest sub-pathway with
#     all of its entities with non-zero IPLs
# 
#     ## Step 6
#     FOR each GO term in GMT file collections:
#         do enrichment analysis using genes from the largest sub-pathway
# 
# cluster samples by their GO enrichment results
# 
# ## Step 7
# pick a sample group of interest
# 
# find submodules shared by the largest sub-pathways of all samples in this group
# 
# identify key pathway proteins from submodules
# 
# ## Step 8
# evaluate the correlation of key protein’s IPLs with patients’ clinical data

## ----installFromGitHub, eval=FALSE-------------------
# devtools::install_github('pliu55/MPAC')

## ----installFromBioconductor, eval=FALSE-------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# # The following initializes usage of Bioc devel
# BiocManager::install(version='devel')
# 
# BiocManager::install("MPAC")

## ----requiredPkg-------------------------------------
require(SummarizedExperiment)
require(MPAC)

## ----ppCnInp-----------------------------------------
# a matrix of CN focal data with rows as genes and columns as samples
cn_tumor_mat <- system.file('extdata/TcgaInp/focal_tumor.rds',
                            package='MPAC') |> readRDS()

# to return a SummarizedExperiment object of CNA state for running PARADIGM
# activated, normal, or repressed state is represented by 1, 0, or -1
ppCnInp(cn_tumor_mat)

## ----ppRnaInp----------------------------------------
# a matrix of RNA-seq data with rows as genes and columns as tumor samples
rna_tumor_mat <- system.file('extdata/TcgaInp/log10fpkmP1_tumor.rds',
                            package='MPAC') |> readRDS()

# a matrix of RNA-seq data with rows as genes and columns as normal samples
rna_norm_mat  <- system.file('extdata/TcgaInp/log10fpkmP1_normal.rds',
                            package='MPAC') |> readRDS()

# to return a SummarizedExperiment object of RNA state for running PARADIGM
# activated, normal, or repressed state is represented by 1, 0, or -1
ppRnaInp(rna_tumor_mat, rna_norm_mat, threads=2)

## ----ppRealInp---------------------------------------
# to return a SummarizedExperiment object of CNA and RNA state 
real_se <- ppRealInp(cn_tumor_mat, rna_tumor_mat, rna_norm_mat, threads=2)

# CNA state is in assays(real_se)$CN_state
# RNA state is in assays(real_se)$RNA_state
real_se

## ----ppPermInp---------------------------------------
# to return a list of list
perml <- ppPermInp(real_se, n_perms=3)

# three objects under the first level
length(perml)

# permutation index
metadata(perml[[1]])$i

# permuted CNA state matrix, same as the one from `ppCnInp()`
assays(perml[[1]])$CN_state |> _[1:4, 1:3]

# permuted RNA state matrix, same as the one from `ppRnaInp()`
assays(perml[[1]])$RNA_state |> _[1:4, 1:3]

## ----echo=FALSE--------------------------------------
prd_paper_url='https://doi.org/10.1093/bioinformatics/btq182'
prd_exe_url = paste0('https://github.com/sng87/paradigm-scripts/tree/',
    'master/public/exe/')
prd_linux_url= paste0(prd_exe_url, 'LINUX')
prd_macos_url= paste0(prd_exe_url, 'MACOSX')

## ----runRealPrd--------------------------------------
# CNA and RNA state from `ppRealInp()`
real_se <- system.file('extdata/TcgaInp/inp_real.rds', package='MPAC') |> 
    readRDS()

# Pathway file
fpth <- system.file('extdata/Pth/tiny_pth.txt', package='MPAC')

# folder to save all the output files
outdir <- tempdir()

# PARADIGM binary location. Replace the one below with a true location.
paradigm_bin <- '/path/to/PARADIGM'

### code below depends on external PARADIGM binary
runPrd(real_se, fpth, outdir, paradigm_bin, sampleids=c('TCGA-CV-7100'))

## ----runPermPrd--------------------------------------
# a list of list from `ppPermInp()`
permll <- system.file('extdata/TcgaInp/inp_perm.rds', package='MPAC') |> 
    readRDS()

# Pathway file
fpth <- system.file('extdata/Pth/tiny_pth.txt', package='MPAC')

# folder to save all the output files
outdir <- tempdir()

# PARADIGM binary location. Replace the one below with a true location.
paradigm_bin <- '/path/to/PARADIGM'

# (optional) sample IDs to run PARADIGM on
pat <- 'TCGA-CV-7100'

### code below depends on external PARADIGM binary
runPermPrd(permll, fpth, outdir, paradigm_bin, sampleids=c(pat))

## ----colRealIPL--------------------------------------
# the folder saving PARADIGM result on real data
# it should be the `outdir` folder from `runPrd()`
indir <- system.file('/extdata/runPrd/', package='MPAC')

# to return a data.table with columns as entities and IPLs for each sample
colRealIPL(indir) |> head()

## ----colPermIPL--------------------------------------
# the folder saving PARADIGM result on permuted data
# it should be the `outdir` folder from `runPermPrd()`
indir <- system.file('/extdata/runPrd/', package='MPAC')

# number of permutated dataset results to collect
n_perms <- 3

# return a data.table with columns as entities, permutation index, and IPLs for
# each sample
colPermIPL(indir, n_perms) |> head()

## ----fltByPerm---------------------------------------
# collected real IPLs. It is the output from `colRealIPL()`
realdt <- system.file('extdata/fltByPerm/real.rds', package='MPAC') |> readRDS()

# collected permutation IPLs. It is the output from `colPermIPL()`
permdt <- system.file('extdata/fltByPerm/perm.rds', package='MPAC') |> readRDS()

# to return a matrix of filtered IPLs with rows as pathway entities and columns
# as samples. Entities with IPLs observed by chance are set to NA.
fltByPerm(realdt, permdt) |> head()

## ----subNtw------------------------------------------
# a matrix generated by `fltByPerm()`
fltmat <- system.file('extdata/fltByPerm/flt_real.rds', package='MPAC') |>
        readRDS()

# a pathway file
fpth <- system.file('extdata/Pth/tiny_pth.txt', package='MPAC')

# a gene set file in MSigDB's GMT format. It should be the same file that will
# be used in the over-representation analysis below.
fgmt <- system.file('extdata/ovrGMT/fake.gmt', package='MPAC')

# to return a list of igraph objects representing the larget sub-pathway for
# each sample
subNtw(fltmat, fpth, fgmt, min_n_gmt_gns=1)

## ----ovrGMT------------------------------------------
# a list of igraph objects from `subNtw()`
subntwl <- system.file('extdata/subNtw/subntwl.rds', package='MPAC') |>readRDS()

# a gene set file that has been used in `subNtw()`
fgmt <- system.file('extdata/ovrGMT/fake.gmt',       package='MPAC')

# (optional) genes that have CN and RNA data in the input files for PARADIGM
omic_gns <- system.file('extdata/TcgaInp/inp_focal.rds', package='MPAC') |>
            readRDS() |> rownames()

# to return a matrix of over-representation adjusted p-values with rows as gene
# set and columns as samples
ovrGMT(subntwl, fgmt, omic_gns)

## ----clSamp------------------------------------------
# a matrix of gene set over-representation adjusted p-values from `ovrGMT()`
ovrmat <- system.file('extdata/clSamp/ovrmat.rds', package='MPAC') |> readRDS()

# to return a data.table of clustering result by 5 random runs:
#
# - each row represents a clustering result
# - the first column, `nreps`, indicates the number of occurrences of a
#   clustering result in the 5 random runs
# - the other columns represents each sample's clustering membership
#
clSamp(ovrmat, n_random_runs=5)

## ----conMtf------------------------------------------
# a list of igraph objects from `subNtw()`
subntwl <- system.file('extdata/conMtf/subntwl.rds', package='MPAC') |>readRDS()

# (optional) genes that have CN and RNA data in the input files for PARADIGM
omic_gns <- system.file('extdata/TcgaInp/inp_focal.rds', package='MPAC') |>
            readRDS() |> rownames()

# to return a list of igraph objects representing consensus motifs
conMtf(subntwl, omic_gns, min_mtf_n_nodes=50)

## ----pltOvrHm----------------------------------------
# GO term adjusted p-values, which are the output of `ovrGMT()`
ovrmat <- system.file('extdata/pltOvrHm/ovr.rds',package='MPAC') |> readRDS()

# clustering information of samples, which is the output of `clSamp()`
cldt   <- system.file('extdata/pltOvrHm/cl.rds', package='MPAC') |> readRDS()

# to plot a heatmap on GO terms significantly over-represented in >= 80% of
# samples in a group
pltOvrHm(ovrmat, cldt, min_frc=0.8)

## ----pltConMtf---------------------------------------
# A list of consensus pathway submodules. It is the output of `conMtf()`
grphl <- system.file('extdata/pltMtfPrtIPL/grphl.rds',package='MPAC') |>
         readRDS()

# Proteins to highlight. Not required--no node in the graph will be highlighted
proteins <- c('CD28', 'CD86', 'LCP2', 'IL12RB1', 'TYK2', 'CD247', 'FASLG', 
              'CD3G')

# To plot consensus pathway submodules
pltConMtf(grphl, proteins)

## ----pltMtfPrtIPL------------------------------------
# A matrix of filtered IPL, which is the output of `fltByPerm()`
fltmat <- system.file('extdata/pltSttKM/ipl.rds', package='MPAC') |> readRDS()

# A data.table of sample clustering results. It is the output of `clSamp()`
cldt <- system.file('extdata/pltMtfPrtIPL/cl.rds',package='MPAC')|> readRDS()

# A list of consensus pathway submodules. It is the output of `conMtf()`
grphl <- system.file('extdata/pltMtfPrtIPL/grphl.rds',package='MPAC') |>
         readRDS()

# Proteins to plot
proteins=c('CD28', 'CD86', 'LCP2', 'IL12RB1', 'TYK2', 'CD247', 'FASLG', 'CD3G')

# To plot a heatmap of IPLs on specified proteins
pltMtfPrtIPL(fltmat, cldt, grphl, proteins)

## ----pltSttKM----------------------------------------
# A matrix containing patient survival information
cdrmat <- system.file('extdata/pltSttKM/cdr.rds', package='MPAC') |> readRDS()

# A matrix of filtered IPL, which is the output of `fltByPerm()`
fltmat <- system.file('extdata/pltSttKM/ipl.rds', package='MPAC') |> readRDS()

# To plot Kaplan-Meier curve using overall survival (OS) event and days from 
# `cdrmat` and pathway states of CD247 and FASLG from `fltmat`. Samples are 
# stratified by whether having both CD247 and FASLG in activated states, i.e., 
# IPL>0 for both proteins.
pltSttKM(cdrmat, fltmat, event='OS', time='OS_days', 
         proteins=c('CD247', 'FASLG'), strat_func='>0')

## ----pltNeiStt---------------------------------------
# protein of focus
protein <- 'CD86'

# input pathway file
fpth <- system.file('extdata/Pth/tiny_pth.txt', package='MPAC')

# CNA and RNA state matrix from `ppRealInp()`
real_se <- system.file('extdata/pltNeiStt/inp_real.rds', package='MPAC') |> 
    readRDS()

# filtered IPL matrix from `fltByPerm()`
fltmat <- system.file('extdata/pltNeiStt/fltmat.rds', package='MPAC') |> 
            readRDS()

# to plot a heatmap
pltNeiStt(real_se, fltmat, fpth, protein)

## ----sessionInfo, echo=FALSE-------------------------
sessionInfo()

#' @title  Plot consensus pathway submodules
#'
#' @param  proteins  A vector of protein symbols to highlight in the plot.
#'                   Default: no protein will be highlighted.
#'
#' @inheritParams pltMtfPrtIPL
#'
#' @return  a plot of consensus pathway submodules
#'
#' @examples
#'
#' grphl <- system.file('extdata/pltMtfPrtIPL/grphl.rds',package='MPAC') |>
#'          readRDS()
#'
#' proteins <- system.file('extdata/TcgaInp/inp_focal.rds', package='MPAC') |>
#'             readRDS() |> rownames() |> c('CD3G')
#'
#' pltConMtf(grphl, proteins) |> print()
#'
#' @export
#'
#' @import igraph
#' @import ggraph
#' @importFrom stringr str_wrap
#'
pltConMtf <- function(grphl, proteins=NULL){
    hasomic = lab = NULL

    ntw = do.call(igraph::union, grphl)
    V(ntw)$hasomic = (V(ntw)$name %in% proteins)
    V(ntw)$lab     = gsub('_', ' ', V(ntw)$name) |> stringr::str_wrap(width=20)

    set.seed(88888888)
    ggraph(ntw, layout='fr') +
    geom_edge_link(width=0.2, color='gray80',
                   arrow=arrow(length=unit(1, 'mm')),
                   start_cap=circle(1, 'mm'), end_cap=circle(1, 'mm')) +
    geom_node_point(aes(color=hasomic), size=1) +
    geom_node_text(aes(label=lab, color=hasomic), repel=TRUE, size=2.5) +
    theme_void() +
    theme( legend.position = 'none' ) +
    scale_color_manual( name   = 'protein with\nomic data',
                        values = c( 'TRUE'  = 'firebrick1',
                                    'FALSE' = 'gray10' ) )
}

main <- function() {
    testPltConMtf()
}

testPltConMtf <- function() {
    grphl = system.file('extdata/pltMtfPrtIPL/grphl.rds',package='MPAC') |>
            readRDS()

    prots = c('CD3G', 'CD86')

    test_that('testPltConMtf', {
        pltConMtf(grphl, prots) |> print() |> expect_no_error()
    })

    fpdf = 'Rplots.pdf'
    if ( file.exists(fpdf) ) file.remove(fpdf)
}

main()

#' @title Prepare input copy-number (CN) alteration and RNA data to run PARADIGM
#'
#' @inheritParams ppCnInp
#' @inheritParams ppRnaInp
#'
#' @return  A SummarizedExperiment object of CN and RNA state for PARADIGM
#'
#' @export
#'
#' @examples
#'
#' fcn = system.file('extdata/TcgaInp/focal_tumor.rds', package='MPAC')
#' ftumor = system.file('extdata/TcgaInp/log10fpkmP1_tumor.rds', package='MPAC')
#' fnorm = system.file('extdata/TcgaInp/log10fpkmP1_normal.rds', package='MPAC')
#'
#' cn_tumor_mat = readRDS(fcn)
#' rna_tumor_mat = readRDS(ftumor)
#' rna_norm_mat  = readRDS(fnorm)
#'
#' ppRealInp(cn_tumor_mat, rna_tumor_mat, rna_norm_mat)
#'
#' @importFrom SummarizedExperiment assays assays<-
#'
ppRealInp <- function(cn_tumor_mat, rna_tumor_mat, rna_normal_mat, rna_n_sd=2,
    threads=1) {
    cn_se <- ppCnInp(cn_tumor_mat)
    rna_se <- ppRnaInp(rna_tumor_mat, rna_normal_mat, rna_n_sd=rna_n_sd,
        threads=threads)

    out_rownames <- intersect(rownames(cn_se), rownames(rna_se))
    out_colnames <- intersect(colnames(cn_se), colnames(rna_se))
    real_se <- cn_se[out_rownames, out_colnames] |> copy()
    assays(real_se)$RNA_state <-
        assays(rna_se)$RNA_state[out_rownames, out_colnames]

    return(real_se)
}

#' @title Prepare input copy-number (CN) alteration data to run PARADIGM
#'
#' @param cn_tumor_mat  A matrix of tumor CN focal data with rows as genes
#'                      and columns as samples. A value of 0 means normal CN,
#'                      > 0 means amplification, and < 0 means deletion.
#'
#' @return  A SummarizedExperiment object of CN state for PARADIGM
#'
#' @export
#'
#' @examples
#'
#' fcn = system.file('extdata/TcgaInp/focal_tumor.rds', package='MPAC')
#' cn_tumor_mat = readRDS(fcn)
#'
#' ppCnInp(cn_tumor_mat)
#'
#' @importFrom SummarizedExperiment SummarizedExperiment assays
#'
ppCnInp <- function(cn_tumor_mat) {
    cn_state_mat <- sign(cn_tumor_mat)
    SummarizedExperiment(assays=list(CN_state=cn_state_mat))
}


#' @title Prepare input RNA data to run PARADIGM
#'
#' @param rna_tumor_mat  A matrix of RNA data from tumor samples with rows as
#'                       genes and columns as samples
#'
#' @param rna_normal_mat  A matrix of RNA data from normal samples with rows
#'                        as genes and columns as samples
#'
#' @param rna_n_sd  Standard deviation range from fitted normal samples to
#'                  define RNA state. Default: 2, i.e. 2*sd
#'
#' @param threads  Number of threads to run in parallel. Default: 1
#'
#' @return A SummarizedExperiment of RNA state for PARADIGM
#'
#' @export
#'
#' @examples
#'
#' ftumor = system.file('extdata/TcgaInp/log10fpkmP1_tumor.rds', package='MPAC')
#' fnorm = system.file('extdata/TcgaInp/log10fpkmP1_normal.rds', package='MPAC')
#' rna_tumor_mat = readRDS(ftumor)
#' rna_norm_mat  = readRDS(fnorm)
#'
#' ppRnaInp(rna_tumor_mat, rna_norm_mat, threads=2)
#'
#' @importFrom SummarizedExperiment SummarizedExperiment assays
#' @importFrom BiocParallel  SnowParam bplapply
#'
ppRnaInp <- function(rna_tumor_mat, rna_normal_mat, rna_n_sd=2, threads=1) {
    out_mat <- getBPPARAM(threads) |>
        bplapply(rownames(rna_normal_mat), fitByGn, rna_normal_mat, BPPARAM=_)|>
        rbindlist() |>
        defState(rna_tumor_mat, rna_n_sd)

    SummarizedExperiment(assays=list(RNA_state=out_mat))
}

#' @import data.table
#'
defState <- function(fitdt, tumor_mat, rna_n_sd=2) {
    norm_mean <- norm_sd <- m_m_nsd <- m_p_nsd <- val <- state <- NULL
    fitdt[, `:=`(
        m_m_nsd = norm_mean - rna_n_sd * norm_sd,
        m_p_nsd = norm_mean + rna_n_sd * norm_sd)]

    t(tumor_mat) |> as.data.table(keep.rownames='brc') |>
    melt(id='brc', variable='gnname', value='val') |>
    merge(fitdt, by='gnname', all.x=TRUE) |>
    _[, state := ifelse(val < m_m_nsd, -1, ifelse(val > m_p_nsd, 1, 0))] |>
    dcast(brc ~ gnname, value.var='state') |>
    _[, c('brc', fitdt$gnname), with=FALSE] |>
    as.matrix(rownames='brc') |> t()
}

#' @importFrom fitdistrplus mgedist
#'
fitByGn <- function(gnname, mat) {
    vec <- mat[gnname, ]
    outl <- list(gnname = gnname)
    scaling_dummy <- 100 ## for fitting on very small numbers
    if ( all(vec == vec[1]) ) {
        outl$norm_mean <- vec[1]
        outl$norm_sd   <- 0
    } else {
        fit <- fitdistrplus::mgedist(vec*scaling_dummy, distr='norm')
        outl$norm_mean <- fit$estimate['mean']/scaling_dummy
        outl$norm_sd   <- fit$estimate['sd']/scaling_dummy
    }

    return(outl)
}


#' @title Permute input genomic state data between genes in the same sample
#'
#' @param real_se  A SummarizedExperiment object of CN and RNA states from
#'                 real samples with rows as genes and columns as samples.
#'                 It is the output from `ppRealInp()`.
#'
#' @param n_perms  Number of permutations. Default: 100
#'
#' @inheritParams ppRnaInp
#'
#' @usage ppPermInp(real_se, n_perms=100, threads=1)
#'
#' @return  A list of SummarizedExperiment objects of permuted CN and RNA
#'          states. The metadata `i` in each obbect denotes its permutation
#'          index.
#'
#' @export
#'
#' @examples
#'
#' freal = system.file('extdata/TcgaInp/inp_real.rds', package='MPAC')
#' real_se = readRDS(freal)
#'
#' ppPermInp(real_se, n_perms=3)
#'
ppPermInp <- function(real_se, n_perms=100, threads=1) {
    ngns <- nrow(real_se)
    permlist <- lapply(seq_len(n_perms),
        function(x) sample(ngns, ngns, replace=FALSE))

    getBPPARAM(threads) |>
    bplapply(seq_len(n_perms), ppByIPerm, permlist, real_se, BPPARAM=_)
}

#'
#' @importFrom S4Vectors metadata metadata<-
#'
ppByIPerm <- function(iperm, permlist, real_se) {
    perms <- permlist[[iperm]]

    perm_se <- real_se[perms, ]
    rownames(perm_se) <- rownames(real_se)
    metadata(perm_se)$i <- iperm

    return(perm_se)
}

suppressMessages(require(SummarizedExperiment))

main <- function() {
    testPpCnInp()
    testPpRnaInp()
    testPpRealInp()

    testPpPermInp()
}

testPpRealInp <- function() {
    fcn = system.file('extdata/TcgaInp/focal_tumor.rds', package='MPAC')
    ftumor = system.file('extdata/TcgaInp/log10fpkmP1_tumor.rds',package='MPAC')
    fnorm = system.file('extdata/TcgaInp/log10fpkmP1_normal.rds',package='MPAC')
    fcmp = system.file('extdata/TcgaInp/inp_real.rds', package='MPAC') 
   
    cn_tumor_mat = readRDS(fcn)
    rna_tumor_mat = readRDS(ftumor)
    rna_norm_mat  = readRDS(fnorm)
   
    out_se = ppRealInp(cn_tumor_mat, rna_tumor_mat, rna_norm_mat)
    cmp_se = readRDS(fcmp)

    test_that('ppRealInp', {expect_identical(out_se, cmp_se)})
}

testPpCnInp <- function() {
    fin  = system.file('extdata/TcgaInp/focal_tumor.rds', package='MPAC')
    fcmp = system.file('extdata/TcgaInp/inp_focal.rds',   package='MPAC')
    out_se = readRDS(fin) |> ppCnInp()

    cmp_se = readRDS(fcmp)

    test_that('ppCnInp', {expect_identical(out_se, cmp_se)})
}

testPpRnaInp <- function() {
    ftumor= system.file('extdata/TcgaInp/log10fpkmP1_tumor.rds', package='MPAC')
    fnorm = system.file('extdata/TcgaInp/log10fpkmP1_normal.rds',package='MPAC')
    fcmp  = system.file('extdata/TcgaInp/inp_log10fpkmP1.rds',   package='MPAC')

    tumor_mat = readRDS(ftumor)
    norm_mat  = readRDS(fnorm)
    out_se = ppRnaInp(tumor_mat, norm_mat)

    cmp_se = readRDS(fcmp)

    test_that('ppRnaInp', {expect_identical(out_se, cmp_se)})
}

testPpPermInp <- function() {
    freal = system.file('extdata/TcgaInp/inp_real.rds', package='MPAC') 
    real_se = readRDS(freal)
    n_perms = 3
    set.seed(123456)
    outll = ppPermInp(real_se, n_perms)

    fcmp = system.file('extdata/TcgaInp/inp_perm.rds', package='MPAC')
    cmpll = readRDS(fcmp)

    lapply(seq_len(n_perms), testPpPermInpByIPerm, outll, cmpll)
}

testPpPermInpByIPerm <- function(iperm, outll, cmpll) {
    func = function(x) x$iperm == iperm
    outlist = Filter(func, outll)
    cmplist = Filter(func, cmpll)

    test_that(paste0('ppPermInp: ', iperm), {
        expect_identical(outlist, cmplist )
    })
}

main()

#' @title  Plot a heatmap of pathway and omic states of a protein and its
#'         pathway neighbors
#'
#' @param  real_se  A SummarizedExperiment object of PARADIGM CNA and RNA
#'                  states. It is the output fromm `ppRealInp()` and must
#'                  contain the omic states for the one defined in the
#'                  `protein` argument.
#'
#' @param  fltmat  A matrix contains filterd IPL with rows as entity and
#'                 column as samples. This is the output from `fltByPerm()`.
#'                 Entity with NA value will be set to 0 and plotted as in
#'                 'normal' state.
#'
#' @param  fpth  Name of a pathway file for PARADIGM.
#'
#' @param  protein  Name of the protein to plot. It requires to have CN and RNA
#'                  state data, as well as pathway data from the input.
#'                  Default: ''
#'
#' @return  A heatmap of pathway and omic states of a protein and its
#'          pathway neighbors
#'
#' @examples
#'
#' fpth = system.file('extdata/Pth/tiny_pth.txt', package='MPAC')
#'
#' freal = system.file('extdata/pltNeiStt/inp_real.rds', package='MPAC')
#' fflt  = system.file('extdata/pltNeiStt/fltmat.rds',   package='MPAC')
#'
#' real_se = readRDS(freal)
#' fltmat = readRDS(fflt)
#' protein = 'CD86'
#'
#' pltNeiStt(real_se, fltmat, fpth, protein)
#'
#' @export
#'
pltNeiStt <- function(real_se, fltmat, fpth, protein='') {
    if ( ! (protein %in% rownames(real_se) ) ) {
        paste0(protein, " is not in the input `real_se`\n") |> stop()
    }
    real_se <- real_se[ protein, ]
    cn_state_mat  <- assays(real_se)$CN_state[ protein, , drop=FALSE]
    rna_state_mat <- assays(real_se)$RNA_state[protein, , drop=FALSE]

    fltmat[is.na(fltmat)] <- 0

    nodedt <- ppNode4PltNeiStt(fpth, protein)
    pltmat <- ppMat4PltNeiStt(cn_state_mat, rna_state_mat, fltmat, nodedt)

    id <- NULL
    plt_nodedt <- nodedt[ id %in% rownames(pltmat) ]

    makeHmNeiStt(pltmat, plt_nodedt)
}

#' @import ComplexHeatmap
#' @importFrom grid  gpar unit
#'
makeHmNeiStt <- function(pltmat, nodedt) {
    level <- type <- id <- row_grp <- NULL
    nodedt <- nodedt[order(level, type, id)]
    nodedt[, row_grp := factor(row_grp, levels=unique(nodedt$row_grp))]
    pltmat <- pltmat[ nodedt$id, ]
    Heatmap(pltmat,
        col = structure(c(MAGENTA, 'gray85', CYAN), names=c(1, 0, -1)),
        rect_gp            = gpar(col='white'),
        border             = 'black',
        row_split          = nodedt$row_grp,
        cluster_row_slices = FALSE,
        row_gap            = unit(0.2, 'cm'),
        row_title_rot      = 0,
        row_title_gp       = gpar(fontsize=FONT_SIZE),
        row_title_side     = 'right',
        cluster_rows       = TRUE,
        cluster_columns    = TRUE,
        clustering_distance_rows    = 'manhattan',
        clustering_distance_columns = 'manhattan',
        show_row_dend      = FALSE,
        show_column_dend   = FALSE,
        show_row_names     = TRUE,
        row_names_side     = 'left',
        show_column_names  = TRUE,
        row_names_gp       = gpar(fontsize=8),
        column_names_gp    = gpar(fontsize=7),
        column_names_side  = 'top',
        column_names_rot   = 30,
        height             = unit(0.4 * (nrow(pltmat)+1), 'cm'),
        width              = unit(0.45 * ncol(pltmat), 'cm'),
        use_raster         = FALSE,
        heatmap_legend_param = list(
            title       = "omic or\npathway state",
            title_gp    = gpar(fontsize=FONT_SIZE),
            labels_gp   = gpar(fontsize=FONT_SIZE),
            grid_height = unit(3.5, 'mm'),
            grid_width  = unit(3.5, 'mm'),
            at          = c(1, 0, -1),
            labels      = c('activated', 'normal', 'repressed'),
            border      = 'black'
        )
    ) |> draw( heatmap_legend_side = 'bottom',
                background = 'transparent' )
}

ppMat4PltNeiStt <- function(cnmat, rnamat, iplmat, nodedt) {
    pats <- intersect(colnames(cnmat), colnames(rnamat)) |>
            intersect(colnames(iplmat)) |> sort()

    type <- NULL
    ents <- intersect(nodedt[ type != 'omic' ]$id, rownames(iplmat))
    pltmat <- rbind(cnmat[, pats], rnamat[, pats], sign(iplmat)[ents, pats])
    rownames(pltmat) <- c('CNA', 'RNA-seq', ents)

    return(pltmat)
}

ppNode4PltNeiStt <- function(fpth, gnname) {
    to <- from <- nei <- entity <- . <- desc <- row_grp <- loc <- NULL
    pthlist <- getNodeEdge(fpth)
    in_nodedt <- pthlist$nodedt
    in_edgedt <- pthlist$edgedt

    omic_edgedt <- copy(OMIC_EDGEDT) |> _[, to := gnname]
    edgedt <- in_edgedt[(from == gnname) | (to %in% gnname)] |>
        rbind(omic_edgedt) |> merge(DASHDT, by='title', all.x=TRUE) |>
        _[, nei := ifelse(from == gnname, to, ifelse(to == gnname, from, NA))]

    up_ents <- edgedt[(to == gnname) & (! from %in% OMIC_NODEDT$entity)]$from
    dn_ents <- edgedt[ from == gnname ]$to

    omic_nodedt <- copy(OMIC_NODEDT)
    outdt <- in_nodedt[entity %in% c(edgedt$from, edgedt$to)] |>
        rbind(omic_nodedt) |> setnames('entity', 'id') |>
        orderNodes(up_ents, dn_ents, gnname) |>
        merge(edgedt[, .(nei, desc)], by.x='id', by.y='nei', all.x=TRUE) |>
        _[, row_grp := ifelse(loc %in% c('omic', gnname), loc,
            paste0(loc, ":\n", desc))]

    return(outdt)
}

orderNodes <- function(nodedt, up_ents, dn_ents, gnname) {
    id <- label <- level <- loc <- NULL
    nodedt[, id := factor(id,
        levels=unique(c(up_ents, dn_ents, 'CNA', gnname, 'RNA-seq')))] |>
    _[, label := gsub('_', ' ', id)] |>
    _[, level := ifelse(id %in% up_ents, 1, ifelse(id %in% dn_ents, 3, 2))] |>
    _[, loc := ifelse(id %in% up_ents, 'upstream',
        ifelse(id %in% dn_ents, 'downstream',
            ifelse(id %in% c('CNA', 'RNA-seq'), 'omic',
                ifelse(id == gnname, gnname, '')))) ]
}

main <- function() {
    testPltNeiStt()
}

testPltNeiStt <- function() {
    fpth = system.file('extdata/Pth/tiny_pth.txt', package='MPAC')

    freal = system.file('extdata/pltNeiStt/inp_real.rds', package='MPAC')
    fflt  = system.file('extdata/pltNeiStt/fltmat.rds',   package='MPAC')

    real_se = readRDS(freal)
    fltmat = readRDS(fflt)
    protein = 'CD86'

    test_that('testPltNeiStt', {
        pltNeiStt(real_se, fltmat, fpth, protein) |>
        expect_no_error()
    })

    fpdf = 'Rplots.pdf'
    if ( file.exists(fpdf) ) file.remove(fpdf)
}

main()

# fake.gmt

A fake GMT file containing two gene sets:

- a gene set named __CD_fake__ that contains genes CD28 and CD86
- a gene set named __MI_fake__ that contains genes CD28 and LCP2


# ovr.rds

```{r}
fntw   = system.file('extdata/subNtw/subntwl.rds',    package='MPAC')
fgmt   = system.file('extdata/ovrGMT/fake.gmt',       package='MPAC')
ffocal = system.file('extdata/TcgaInp/inp_focal.rds', package='MPAC')

subntwl  = readRDS(fntw)
omic_gns = readRDS(ffocal) |> rownames()

MPAC::ovrGMT(subntwl, fgmt, omic_gns)
```

# Data downloaded from TCGA

## focal_tumor.rds  

This file contains gene-level copy-number scores of TCGA-HNSC HPV+ patients 
samples downloaded from [NCI GDC Data Portal](https://portal.gdc.cancer.gov/) 
version 29. 

Data in this file is an R matrix object with rows for seven genes and columns 
for 71 patients.

## log10fpkmP1_normal.rds  

This file contains RNA expression levels in $log_{10}(FPKM+1)$ of TCGA-HNSC 
_normal_ samples downloaded from the same source as the copy-number data above. 

Data in this file is an R matrix object with rows for the same seven genes as 
copy-number data above and columns for 44 patients.

## log10fpkmP1_tumor.rds

This file contains RNA expression levels in $log_{10}(FPKM+1)$ of TCGA-HNSC 
_primary tumor_ samples downloaded from the same source as the copy-number data 
above. 

Data in this file is an R matrix object with rows for seven genes and columns 
for 71 patients. Both genes and patients are the same as the copy-number
data above.


# Data derived from the download data

## inp_focal.rds

```{r}
system.file('extdata/TcgaInp/focal_tumor.rds', package='MPAC') |> readRDS() |> 
MPAC::ppCnInp()
```

## inp_log10fpkmP1.rds    

```{r}
ftumor = system.file('extdata/TcgaInp/log10fpkmP1_tumor.rds',  package='MPAC')
fnorm  = system.file('extdata/TcgaInp/log10fpkmP1_normal.rds', package='MPAC')

tumor_mat = readRDS(ftumor)
norm_mat  = readRDS(fnorm)

MPAC::ppRnaInp(tumor_mat, norm_mat)
```

## inp_real.rds     

```{r}
fcn    = system.file('extdata/TcgaInp/focal_tumor.rds',        package='MPAC')
ftumor = system.file('extdata/TcgaInp/log10fpkmP1_tumor.rds',  package='MPAC')
fnorm  = system.file('extdata/TcgaInp/log10fpkmP1_normal.rds', package='MPAC')

cn_tumor_mat  = readRDS(fcn)
rna_tumor_mat = readRDS(ftumor)
rna_norm_mat  = readRDS(fnorm)

MPAC::ppRealInp(cn_tumor_mat, rna_tumor_mat, rna_norm_mat) 
```

## inp_perm.rds

```{r}
set.seed(123456)

system.file('extdata/TcgaInp/inp_real.rds', package='MPAC') |> readRDS() |> 
MPAC::ppPermInp(n_perms=3)
```

# real.rds

```{r}
indir = system.file('/extdata/runPrd/', package='MPAC')
pat = 'TCGA-CV-7100'

MPAC::colRealIPL(indir, sampleids=c(pat))
```

# perm.rds

```{r}
indir = system.file('/extdata/runPrd/', package='MPAC')
n_perms = 3
pat = 'TCGA-CV-7100'

MPAC::colPermIPL(indir, n_perms, sampleids=c(pat))
```

# Prediction on real data

```{r}
freal = system.file('extdata/TcgaInp/inp_real.rds', package='MPAC')
fpth  = system.file('extdata/Pth/tiny_pth.txt',     package='MPAC')

real_se = readRDS(freal)
pat = 'TCGA-CV-7100'

outdir   = '/output/folder/'
PARADIGM = '/location/PARADIGM'
MPAC::runPrd(real_se, fpth, outdir, PARADIGM_bin=PARADIGM, sampleids=c(pat))
```

The R code above will generate the following files in `outdir` (remember to 
supply the location of PARADIGM above):

- TCGA-CV-7100_cfg.txt
- TCGA-CV-7100_cpt.txt
- TCGA-CV-7100_inp_focal.tsv
- TCGA-CV-7100_inp_log10fpkmP1.tsv
- TCGA-CV-7100_ipl.txt


# Prediction on permuted data

```{r}
fpermll = system.file('extdata/TcgaInp/inp_perm.rds', package='MPAC')
fpth    = system.file('extdata/Pth/tiny_pth.txt',     package='MPAC')

permll = readRDS(fpermll)
pat = 'TCGA-CV-7100'

outdir   = '/output/folder/'
PARADIGM = '/location/PARADIGM'

MPAC::runPermPrd(permll, fpth, outdir, PARADIGM_bin=PARADIGM, sampleids=c(pat))
```

The R code above will generate the same set of files as in __Prediction on 
real data__. The output files will be in the following folders (again, remember
to supply the location of PARADIGM when running prediction):

- `outdir/p1/`
- `outdir/p2/`
- `outdir/p3/`

# subntwl.rds

This file contains a subset of pathway network for two TCGA-HNSC HPV+ patients.
For more details, please see this package's CITATION.

Data in this file is a list of two igraph objects for patient TCGA-MZ-A5BI and
TCGA-QK-A6IF, belonging to the 'immune response' group described in this 
package's CITATION. Both objects contain the seven key proteins, CD28, CD86, 
LCP2, IL12RB1, TYK2, CD247, and FASLG.


# con_grph.rds

```{r}
subntwl = system.file('extdata/conMtf/subntwl.rds', package='MPAC') |> readRDS()

suppressMessages(require(SummarizedExperiment))
omic_gns = system.file('extdata/TcgaInp/inp_focal.rds', package='MPAC') |>
           readRDS() |> rownames()

MPAC::conMtf(subntwl, omic_gns, min_mtf_n_nodes=50)
```

# real.rds

This file contains inferred pathway levels from _real_ copy-number and RNA 
input. For more details, please see this package's CITATION.

Data in this file is a data.table object with rows corresponding to 30 pathway
entities and columns for entity's identity and inferred pathway levels for 
patient TCGA-BA-5152 and TCGA-CV-7100.


# perm.rds

This file contains inferred pathway levels from _permuted_ copy-number and RNA
input. For more details, please see this package's CITATION.

Data in this file is a data.table object with rows corresponding to 30 pathway
entities from 100 sets of permuted inputs. Columns are for entity's identity, 
permutation index, and inferred pathway levels for patient TCGA-BA-5152 and 
TCGA-CV-7100. 


# flt_real.rds

```{r}
realdt = system.file('extdata/fltByPerm/real.rds', package='MPAC') |> readRDS()
permdt = system.file('extdata/fltByPerm/perm.rds', package='MPAC') |> readRDS()

MPAC::fltByPerm(realdt, permdt)
```

# subntwl.rds

```{r}
fflt = system.file('extdata/fltByPerm/flt_real.rds', package='MPAC')
fpth = system.file('extdata/Pth/tiny_pth.txt',       package='MPAC')
fgmt = system.file('extdata/ovrGMT/fake.gmt',        package='MPAC')

fltdt = readRDS(fflt)
MPAC::subNtw(fltdt, fpth, fgmt, min_n_gmt_gns=1)
```

# ovrmat.rds

This file contains GO enrichment results for 18 TCGA-HNSC HPV+ patients. For 
more details, please see this package's CITATION.

Data in this file is an R matrix object with rows corresponding to 2085 GO terms
and columns for the 18 patients. This matrix saves adjusted p-values for GO 
enrichment.


# clmat.rds

```{r}
outmat = system.file('extdata/clSamp/ovrmat.rds', package='MPAC') |> 
         readRDS() |> MPAC::clSamp(n_neighbors=10, n_random_runs=5) |> 
         _[order(-nreps)] |> as.matrix(rownames='nreps') |> _[1, ] |> t() |> t()

colnames(outmat) = 'icl'

outmat
```

# tiny_pth.txt

A tiny subset of the pathway network from 
[the TCGA Pan-Cancer Atlas](https://doi.org/10.1016/j.cell.2018.03.022). It 
focuses on proteins TYK2, IL12RB1, CD28, CD86, LCP2, CD247, and FASLG, 
as well as entities connecting to them directly.

---
title: 'MPAC: Multi-omic Pathway Analysis of Cells'
author: 'Peng Liu, David Page, Paul Ahlquist, Irene M. Ong, and Anthony Gitter'
date: '`r Sys.Date()`'
output:
    bookdown::html_document2:
        toc: true
        fig_caption: yes
        dev: svglite
        toc_float: true
urlcolor: blue
linkcolor: blue
vignette: >
    %\VignetteIndexEntry{MPAC: Multi-omic Pathway Analysis of Cells}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set( cache = TRUE, collapse = TRUE, echo = TRUE,
    message = FALSE, warning = FALSE, fig.align = 'center',
    dev.args=list(fix_text_size=FALSE ), comment = "#>")

options( htmltools.dir.version = FALSE, formatR.indent = 2,
    width = 55, digits = 4, warnPartialMatchAttr = FALSE,
    warnPartialMatchDollar = FALSE, tinytex.verbose=TRUE)

require(data.table)

require(MPAC)
```

# Introduction

From copy-number alteration (CNA), RNA-seq data, and pre-defined biological
pathway network, MPAC computes Inferred Pathway Levels (IPLs) for pathway 
entities, clusters patient samples by their enriched GO terms, and identifies
key pathway proteins for each patient cluster.

## Workflow

```{r WorkflowJPG, fig.cap='MPAC workflow', out.width='800px', echo=FALSE}
knitr::include_graphics('workflow.jpg')
```

## Pseudocode

```{r pseudocode, eval=FALSE}
FOR each sample:
    ## Step 1
    prepare CNA states from real data

    prepare RNA states from real data

    ## Step 2
    compute IPLs by PARADIGM using CNA and RNA states from real data as well as
    the input pathway definition

    ## Step 3
    REPEAT 100 times:
        randomly permute CNA and RNA states between genes

        compute IPLs by PARADIGM using CNA and RNA states from permuted data as
        well as the input pathway definition

    ## Step 4
    FOR each pathway entity:
        filter entity’s IPL from real data by its 100 IPLs from permuted data

    ## Step 5
    based on the input pathway definition, select the largest sub-pathway with 
    all of its entities with non-zero IPLs

    ## Step 6
    FOR each GO term in GMT file collections:
        do enrichment analysis using genes from the largest sub-pathway

cluster samples by their GO enrichment results

## Step 7
pick a sample group of interest

find submodules shared by the largest sub-pathways of all samples in this group

identify key pathway proteins from submodules

## Step 8
evaluate the correlation of key protein’s IPLs with patients’ clinical data
```

# Installation

## From GitHub

Start R and enter:

```{r installFromGitHub, eval=FALSE}
devtools::install_github('pliu55/MPAC')
```

## From Bioconductor

Start R and enter:

```{r installFromBioconductor, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

# The following initializes usage of Bioc devel
BiocManager::install(version='devel')

BiocManager::install("MPAC")
```

For different versions of R, please refer to the appropriate
[Bioconductor release](https://bioconductor.org/about/release-announcements/).


# Required R packages for this vignette

Please use the following code to load required packages for running this
vignette. Many intermediate objects in this vignette are SummarizedExperiment
objects.

```{r requiredPkg}
require(SummarizedExperiment)
require(MPAC)
```

# Main functions

- `ppCnInp()`: prepare CNA states from real CNA data
- `ppRnaInp()`: prepare RNA states from read RNA-seq data
- `ppRealInp()`: prepare CNA and RNA states from read CNA and RNA-seq data
- `ppPermInp():` prepare permuted CNA and RNA states 
- `runPrd()`: run PARADIGM to compute IPLs from real CNA and RNA states
- `runPermPrd()`: run PARADIGM to compute IPLs from permuted CNA and RNA states
- `colRealIPL()`: collect IPLs from real data
- `colPermIPL()`: collect IPLs from permuted data
- `fltByPerm()`: filter IPLs from real data by IPLs from permuted data
- `subNtw()`: find the largest pathway network subset by filtered IPLs
- `ovrGMT()`: do gene set over-representation on a sample's largest pathway
    subset
- `clSamp()`: cluster samples by their gene set over-representation results
- `conMtf()`: find consensus pathway submoduless within a cluster
- `pltConMtf()`: plot consensus pathway submodules
- `pltMtfPrtIPL()`: plot a heatmap of IPLs for proteins from consensus pathway
                  submodule(s)
- `pltSttKM()`: plot a Kaplan-Meier curve to find correlation between 
              protein(s)' pathway states and patient survival
- `pltNeiStt()`: plot a heatmap to identify a protein's pathway determinants
- `pltOvrHm()`: plot a heatmap to identify over-represented GO terms for each 
              sample groups



# Preparing input

## Requirements

`r focal_url=paste0('https://docs.gdc.cancer.gov/Data/',
    'Bioinformatics_Pipelines/CNV_Pipeline/#data-processing-steps_2')`

MPAC accepts two types of genomic data: 

- copy-number alteration (CNA): recommend using data from
    [focal-level](`r focal_url`).
- RNA-seq: both tumor and normal sample data are required. Recommend using 
    normalized gene expression data, e.g. FPKM.

## Real data

This section describes preparing PARADIGM input using data from real samples,
which can be downloaded from TCGA or taken from sequencing experiments.

### Copy-number alteration

```{r ppCnInp}
# a matrix of CN focal data with rows as genes and columns as samples
cn_tumor_mat <- system.file('extdata/TcgaInp/focal_tumor.rds',
                            package='MPAC') |> readRDS()

# to return a SummarizedExperiment object of CNA state for running PARADIGM
# activated, normal, or repressed state is represented by 1, 0, or -1
ppCnInp(cn_tumor_mat)
```

### RNA-seq

Note that data from normal samples are required here to evaluate if a gene
in tumor samples has activated, repressed, or normal states.

```{r ppRnaInp}
# a matrix of RNA-seq data with rows as genes and columns as tumor samples
rna_tumor_mat <- system.file('extdata/TcgaInp/log10fpkmP1_tumor.rds',
                            package='MPAC') |> readRDS()

# a matrix of RNA-seq data with rows as genes and columns as normal samples
rna_norm_mat  <- system.file('extdata/TcgaInp/log10fpkmP1_normal.rds',
                            package='MPAC') |> readRDS()

# to return a SummarizedExperiment object of RNA state for running PARADIGM
# activated, normal, or repressed state is represented by 1, 0, or -1
ppRnaInp(rna_tumor_mat, rna_norm_mat, threads=2)
```

### Copy-number alteration and RNA-seq

Simply use a wrapper function below to prepare input together for both CNA
and RNA data

```{r ppRealInp}
# to return a SummarizedExperiment object of CNA and RNA state 
real_se <- ppRealInp(cn_tumor_mat, rna_tumor_mat, rna_norm_mat, threads=2)

# CNA state is in assays(real_se)$CN_state
# RNA state is in assays(real_se)$RNA_state
real_se
```


## Permuted data

Pathway activities from permuted data will be used to build a background 
distribution to filter those from real data. The idea is to remove those that
could be observed by chance. Permuted data were from real CNA and RNA data as 
shown in the example below.

```{r ppPermInp}
# to return a list of list
perml <- ppPermInp(real_se, n_perms=3)

# three objects under the first level
length(perml)

# permutation index
metadata(perml[[1]])$i

# permuted CNA state matrix, same as the one from `ppCnInp()`
assays(perml[[1]])$CN_state |> _[1:4, 1:3]

# permuted RNA state matrix, same as the one from `ppRnaInp()`
assays(perml[[1]])$RNA_state |> _[1:4, 1:3]
```


# Inferred pathway levels

## PARADIGM binary

```{r, echo=FALSE}
prd_paper_url='https://doi.org/10.1093/bioinformatics/btq182'
prd_exe_url = paste0('https://github.com/sng87/paradigm-scripts/tree/',
    'master/public/exe/')
prd_linux_url= paste0(prd_exe_url, 'LINUX')
prd_macos_url= paste0(prd_exe_url, 'MACOSX')
```

MPAC uses PARADIGM developed by [Vaske et al.](`r prd_paper_url`) to predict
pathway levels. PARADIGM Binary is available to download at Github for 
[Linux](`r prd_linux_url`) and [MacOS](`r prd_macos_url`).

## IPLs from real data

Example below shows input and output files for running PARADIGM on data from
real samples.

PARADIGM will generate multiple output files. All the file names will start with
the sample ID and a suffix indicating their types:

- Inferred pathway levles (IPLs): file with a suffix `ipl.txt`
- Output log: file with a suffix `run.out`
- Error log: file with a suffix `run.err`
- Other auxiliary files, which can be skipped under common usage

```{r runRealPrd}
# CNA and RNA state from `ppRealInp()`
real_se <- system.file('extdata/TcgaInp/inp_real.rds', package='MPAC') |> 
    readRDS()

# Pathway file
fpth <- system.file('extdata/Pth/tiny_pth.txt', package='MPAC')

# folder to save all the output files
outdir <- tempdir()

# PARADIGM binary location. Replace the one below with a true location.
paradigm_bin <- '/path/to/PARADIGM'

### code below depends on external PARADIGM binary
runPrd(real_se, fpth, outdir, paradigm_bin, sampleids=c('TCGA-CV-7100'))
```

## PARADIGM on permuted data

For permuted input, PARADIGM will generate output in the same fashion as on
real data above, except that one permutation corresponds to one output folder
named as `p$(index)`, where `$index` is the index of that permutation. For 
example, three permutations will generate folders `p1`, `p2`, and `p3`.

```{r runPermPrd}
# a list of list from `ppPermInp()`
permll <- system.file('extdata/TcgaInp/inp_perm.rds', package='MPAC') |> 
    readRDS()

# Pathway file
fpth <- system.file('extdata/Pth/tiny_pth.txt', package='MPAC')

# folder to save all the output files
outdir <- tempdir()

# PARADIGM binary location. Replace the one below with a true location.
paradigm_bin <- '/path/to/PARADIGM'

# (optional) sample IDs to run PARADIGM on
pat <- 'TCGA-CV-7100'

### code below depends on external PARADIGM binary
runPermPrd(permll, fpth, outdir, paradigm_bin, sampleids=c(pat))
```


# Collecting IPLs

MPAC has PARADIGM run on individual sample in parallel to speed up calculation.
For the convenience of downstream analysis, PARADIGM results will be collected
and put together for all samples. 

## From PARADIGM on real data

```{r colRealIPL}
# the folder saving PARADIGM result on real data
# it should be the `outdir` folder from `runPrd()`
indir <- system.file('/extdata/runPrd/', package='MPAC')

# to return a data.table with columns as entities and IPLs for each sample
colRealIPL(indir) |> head()
```

## From PARADIGM on permuted data

```{r colPermIPL}
# the folder saving PARADIGM result on permuted data
# it should be the `outdir` folder from `runPermPrd()`
indir <- system.file('/extdata/runPrd/', package='MPAC')

# number of permutated dataset results to collect
n_perms <- 3

# return a data.table with columns as entities, permutation index, and IPLs for
# each sample
colPermIPL(indir, n_perms) |> head()
```


# Filtering IPLs

MPAC uses PARADIGM runs on permuted data to generate a background distribution
of IPLs. This distribution is used to filter IPLs from real data to remove
those could be observed by chance. 

```{r fltByPerm}
# collected real IPLs. It is the output from `colRealIPL()`
realdt <- system.file('extdata/fltByPerm/real.rds', package='MPAC') |> readRDS()

# collected permutation IPLs. It is the output from `colPermIPL()`
permdt <- system.file('extdata/fltByPerm/perm.rds', package='MPAC') |> readRDS()

# to return a matrix of filtered IPLs with rows as pathway entities and columns
# as samples. Entities with IPLs observed by chance are set to NA.
fltByPerm(realdt, permdt) |> head()
```

# Find the largest pathway subset

MPAC decomposes the original pathway and identify the largest pathway subset 
with all of its entities having filtered IPLs. This sub-pathway allows the user
to focus on the most altered pathway network. Note that, because the set of 
entities having filtered IPLs are often different between samples, the 
pathway subset will be different between samples as well and represent 
sample-specific features.

```{r subNtw}
# a matrix generated by `fltByPerm()`
fltmat <- system.file('extdata/fltByPerm/flt_real.rds', package='MPAC') |>
        readRDS()

# a pathway file
fpth <- system.file('extdata/Pth/tiny_pth.txt', package='MPAC')

# a gene set file in MSigDB's GMT format. It should be the same file that will
# be used in the over-representation analysis below.
fgmt <- system.file('extdata/ovrGMT/fake.gmt', package='MPAC')

# to return a list of igraph objects representing the larget sub-pathway for
# each sample
subNtw(fltmat, fpth, fgmt, min_n_gmt_gns=1)
```

# Gene set over-representation

To understand the biological functions of a sample's largest sub-pathway, MPAC
performs gene set over-representation analysis on genes with non-zero IPLs in 
the sub-pathway. 

```{r ovrGMT}
# a list of igraph objects from `subNtw()`
subntwl <- system.file('extdata/subNtw/subntwl.rds', package='MPAC') |>readRDS()

# a gene set file that has been used in `subNtw()`
fgmt <- system.file('extdata/ovrGMT/fake.gmt',       package='MPAC')

# (optional) genes that have CN and RNA data in the input files for PARADIGM
omic_gns <- system.file('extdata/TcgaInp/inp_focal.rds', package='MPAC') |>
            readRDS() |> rownames()

# to return a matrix of over-representation adjusted p-values with rows as gene
# set and columns as samples
ovrGMT(subntwl, fgmt, omic_gns)
```

# Cluster samples by pathway over-representation

With gene set over-representation adjusted p-values, MPAC can cluster samples
as a way to investigate shared features between samples in the same cluster.

Note that, due to randomness introduced in the louvain clustering in igraph R 
package version 1.3 (reported in its 
[Github issue #539](https://github.com/igraph/rigraph/issues/539)), it is 
recommended to run clustering multiple times to evaluate its variation. The 
`clSamp()` function has an argument, `n_random_runs`, to specify the number of
random clustering jobs to run.

```{r clSamp}
# a matrix of gene set over-representation adjusted p-values from `ovrGMT()`
ovrmat <- system.file('extdata/clSamp/ovrmat.rds', package='MPAC') |> readRDS()

# to return a data.table of clustering result by 5 random runs:
#
# - each row represents a clustering result
# - the first column, `nreps`, indicates the number of occurrences of a
#   clustering result in the 5 random runs
# - the other columns represents each sample's clustering membership
#
clSamp(ovrmat, n_random_runs=5)
```


# Find consensus pathway motifs within a cluster

From the clustering results, MPAC can find consensus pathway motifs from samples
within the same clusters. These motifs will represent cluster-specific pathway
features. They often contain key proteins for further analysis.

```{r conMtf}
# a list of igraph objects from `subNtw()`
subntwl <- system.file('extdata/conMtf/subntwl.rds', package='MPAC') |>readRDS()

# (optional) genes that have CN and RNA data in the input files for PARADIGM
omic_gns <- system.file('extdata/TcgaInp/inp_focal.rds', package='MPAC') |>
            readRDS() |> rownames()

# to return a list of igraph objects representing consensus motifs
conMtf(subntwl, omic_gns, min_mtf_n_nodes=50)
```

# Diagnostic plots

## Clustered samples by their altered pathways

MPAC provides a function to plot a heatmap of samples clustered by their 
significantly over-represented GO terms

```{r pltOvrHm}
#| fig.dim = c(8, 3.5),
#| fig.cap = "A heatmap of samples clustered by their altered pathways"

# GO term adjusted p-values, which are the output of `ovrGMT()`
ovrmat <- system.file('extdata/pltOvrHm/ovr.rds',package='MPAC') |> readRDS()

# clustering information of samples, which is the output of `clSamp()`
cldt   <- system.file('extdata/pltOvrHm/cl.rds', package='MPAC') |> readRDS()

# to plot a heatmap on GO terms significantly over-represented in >= 80% of
# samples in a group
pltOvrHm(ovrmat, cldt, min_frc=0.8)
```

## Visualize consensus pathway submodules with proteins highlighted

MPAC provides a function to visualize consensus pathway submodules and highlight
the proteins user-supplied.


```{r pltConMtf}
#| fig.dim = c(7, 5),
#| fig.cap = "Consensus pathway submodules."

# A list of consensus pathway submodules. It is the output of `conMtf()`
grphl <- system.file('extdata/pltMtfPrtIPL/grphl.rds',package='MPAC') |>
         readRDS()

# Proteins to highlight. Not required--no node in the graph will be highlighted
proteins <- c('CD28', 'CD86', 'LCP2', 'IL12RB1', 'TYK2', 'CD247', 'FASLG', 
              'CD3G')

# To plot consensus pathway submodules
pltConMtf(grphl, proteins)
```


## Heatmap of IPLs on proteins from consensus pathway submodule(s)

MPAC provides a function to plot heatmap of IPLs on proteins from consensus
pathway submodules. This help to identify proteins with consistent activated,
normal, or repressed pathway states in a cluster of samples.


```{r pltMtfPrtIPL}
#| fig.dim = c(7, 3.5),
#| fig.cap = "Heatmap of IPLs on proteins from consensus pathway submodules."

# A matrix of filtered IPL, which is the output of `fltByPerm()`
fltmat <- system.file('extdata/pltSttKM/ipl.rds', package='MPAC') |> readRDS()

# A data.table of sample clustering results. It is the output of `clSamp()`
cldt <- system.file('extdata/pltMtfPrtIPL/cl.rds',package='MPAC')|> readRDS()

# A list of consensus pathway submodules. It is the output of `conMtf()`
grphl <- system.file('extdata/pltMtfPrtIPL/grphl.rds',package='MPAC') |>
         readRDS()

# Proteins to plot
proteins=c('CD28', 'CD86', 'LCP2', 'IL12RB1', 'TYK2', 'CD247', 'FASLG', 'CD3G')

# To plot a heatmap of IPLs on specified proteins
pltMtfPrtIPL(fltmat, cldt, grphl, proteins)
```

## Correlation between protein(s) pathway states and patient survival

MPAC provides a function to plot Kaplan-Meier curve on patient samples 
stratified by pathway states of one or multiple proteins.

```{r pltSttKM}
#| fig.dim = c(3.5, 3.5),
#| fig.cap = "Kaplan-Meier curve of patient samples stratified by whether 
#|            IPLs of CD247 and FASLG are both > 0."

# A matrix containing patient survival information
cdrmat <- system.file('extdata/pltSttKM/cdr.rds', package='MPAC') |> readRDS()

# A matrix of filtered IPL, which is the output of `fltByPerm()`
fltmat <- system.file('extdata/pltSttKM/ipl.rds', package='MPAC') |> readRDS()

# To plot Kaplan-Meier curve using overall survival (OS) event and days from 
# `cdrmat` and pathway states of CD247 and FASLG from `fltmat`. Samples are 
# stratified by whether having both CD247 and FASLG in activated states, i.e., 
# IPL>0 for both proteins.
pltSttKM(cdrmat, fltmat, event='OS', time='OS_days', 
         proteins=c('CD247', 'FASLG'), strat_func='>0')
```

## Display omic and pathway states of a protein and its neighbors

MPAC implemented a diagnostic function to plot a heatmap of the omic and 
pathway states of a protein as well as the pathway states of this protein's 
pathway neighbors. This heatmap facilitates the identification of pathway 
determinants of this protein.

```{r pltNeiStt}
#| fig.dim = c(8, 3.5),
#| fig.cap = "A heatmap of CD86's omic and pathway state as well as its pathway
#|            neighbor's states."

# protein of focus
protein <- 'CD86'

# input pathway file
fpth <- system.file('extdata/Pth/tiny_pth.txt', package='MPAC')

# CNA and RNA state matrix from `ppRealInp()`
real_se <- system.file('extdata/pltNeiStt/inp_real.rds', package='MPAC') |> 
    readRDS()

# filtered IPL matrix from `fltByPerm()`
fltmat <- system.file('extdata/pltNeiStt/fltmat.rds', package='MPAC') |> 
            readRDS()

# to plot a heatmap
pltNeiStt(real_se, fltmat, fpth, protein)
```

# Session Info

Below is the output of `sessionInfo()` on the system on which this document
was compiled.

```{r sessionInfo, echo=FALSE}
sessionInfo()
```

# inp_real.rds

This file contains the copy-number and RNA state for CD86 in the eleven 
TCGA-HNSC HPV+ patients in the 'immune response' group. For more details, 
please see this package's CITATION.

Data in this file is a SummarizedExperiment object with one row for CD86 and 
eleven columns for patient samples. It contains two assays for copy-number
and RNA state.


# fltmat.rds

This file contains the inferred pathway levels for CD86 and its pathway 
neighbors in the eleven TCGA-HNSC HPV+ patients in the 'immune response' 
group. For more details, please see this package's CITATION.

Data in this file is an R matrix object with seven rows for CD86 and its 
directly connected pathway network neighbors. This matrix has eleven columns 
for patient samples. Values in this matrix are inferred pathway levels.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ovrGMT.R
\name{ovrGMT}
\alias{ovrGMT}
\title{Calculate over-representation of gene sets in each sample by genes
from sample's largest sub-pathway}
\usage{
ovrGMT(subntwlist, fgmt, omic_genes = NULL, threads = 1)
}
\arguments{
\item{subntwlist}{A list of igraph objects represented the largest
sub-pathway for each sample. It is the output of
\code{subNtw()}.}

\item{fgmt}{A gene set GMT file. This will be the same file used for the
gene set over-representation calculation in the next step. It
is used here to ensure output sub-pathway contains a minimum
number of genes from to-be-used gene sets.}

\item{omic_genes}{A vector of gene symbols to narrow down
over-representation calculation to only those with input
genomic data. If not provided, all genes in the GMT file
will be considered. Default: NULL.}

\item{threads}{Number of threads to run in parallel. Default: 1}
}
\value{
A matrix containing over-representation adjusted P with rows as
gene set names and columns as sample IDs.
}
\description{
Calculate over-representation of gene sets in each sample by genes
from sample's largest sub-pathway
}
\examples{

fsubntwl  = system.file('extdata/subNtw/subntwl.rds',    package='MPAC')
fgmt      = system.file('extdata/ovrGMT/fake.gmt',       package='MPAC')
fomic_gns = system.file('extdata/TcgaInp/inp_focal.rds', package='MPAC')
subntwl  = readRDS(fsubntwl)
omic_gns = rownames(readRDS(fomic_gns))

ovrGMT(subntwl, fgmt, omic_gns)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ppInp.R
\name{ppRealInp}
\alias{ppRealInp}
\title{Prepare input copy-number (CN) alteration and RNA data to run PARADIGM}
\usage{
ppRealInp(
  cn_tumor_mat,
  rna_tumor_mat,
  rna_normal_mat,
  rna_n_sd = 2,
  threads = 1
)
}
\arguments{
\item{cn_tumor_mat}{A matrix of tumor CN focal data with rows as genes
and columns as samples. A value of 0 means normal CN,
> 0 means amplification, and < 0 means deletion.}

\item{rna_tumor_mat}{A matrix of RNA data from tumor samples with rows as
genes and columns as samples}

\item{rna_normal_mat}{A matrix of RNA data from normal samples with rows
as genes and columns as samples}

\item{rna_n_sd}{Standard deviation range from fitted normal samples to
define RNA state. Default: 2, i.e. 2*sd}

\item{threads}{Number of threads to run in parallel. Default: 1}
}
\value{
A SummarizedExperiment object of CN and RNA state for PARADIGM
}
\description{
Prepare input copy-number (CN) alteration and RNA data to run PARADIGM
}
\examples{

fcn = system.file('extdata/TcgaInp/focal_tumor.rds', package='MPAC')
ftumor = system.file('extdata/TcgaInp/log10fpkmP1_tumor.rds', package='MPAC')
fnorm = system.file('extdata/TcgaInp/log10fpkmP1_normal.rds', package='MPAC')

cn_tumor_mat = readRDS(fcn)
rna_tumor_mat = readRDS(ftumor)
rna_norm_mat  = readRDS(fnorm)

ppRealInp(cn_tumor_mat, rna_tumor_mat, rna_norm_mat)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ppInp.R
\name{ppPermInp}
\alias{ppPermInp}
\title{Permute input genomic state data between genes in the same sample}
\usage{
ppPermInp(real_se, n_perms=100, threads=1)
}
\arguments{
\item{real_se}{A SummarizedExperiment object of CN and RNA states from
real samples with rows as genes and columns as samples.
It is the output from \code{ppRealInp()}.}

\item{n_perms}{Number of permutations. Default: 100}

\item{threads}{Number of threads to run in parallel. Default: 1}
}
\value{
A list of SummarizedExperiment objects of permuted CN and RNA
states. The metadata \code{i} in each obbect denotes its permutation
index.
}
\description{
Permute input genomic state data between genes in the same sample
}
\examples{

freal = system.file('extdata/TcgaInp/inp_real.rds', package='MPAC')
real_se = readRDS(freal)

ppPermInp(real_se, n_perms=3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pltConMtf.R
\name{pltConMtf}
\alias{pltConMtf}
\title{Plot consensus pathway submodules}
\usage{
pltConMtf(grphl, proteins = NULL)
}
\arguments{
\item{grphl}{A list of igraph objects representing consensus pathway
submodules. It is the output from \code{conMtf()}.}

\item{proteins}{A vector of protein symbols to highlight in the plot.
Default: no protein will be highlighted.}
}
\value{
a plot of consensus pathway submodules
}
\description{
Plot consensus pathway submodules
}
\examples{

grphl <- system.file('extdata/pltMtfPrtIPL/grphl.rds',package='MPAC') |>
         readRDS()

proteins <- system.file('extdata/TcgaInp/inp_focal.rds', package='MPAC') |>
            readRDS() |> rownames() |> c('CD3G')

pltConMtf(grphl, proteins) |> print()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fltByPerm.R
\name{fltByPerm}
\alias{fltByPerm}
\title{Filter IPLs from real data by distribution from permuted data}
\usage{
fltByPerm(realdt, permdt, threads = 1)
}
\arguments{
\item{realdt}{A data.table object containing entities and their IPLs from
real data. It is the output from \code{colRealIPL()}.}

\item{permdt}{A data.table object containing permutation index, entities
and their IPLs from permuted data. It is the output from
\code{colPermIPL()}.}

\item{threads}{Number of threads to run in parallel. Default: 1}
}
\value{
A matrix of filtered IPLs with rows as entities and columns as
samples. Entities with IPLs observed by chance are set to NA.
}
\description{
Filter IPLs from real data by distribution from permuted data
}
\examples{

freal = system.file('extdata/fltByPerm/real.rds', package='MPAC')
fperm = system.file('extdata/fltByPerm/perm.rds', package='MPAC')
realdt = readRDS(freal)
permdt = readRDS(fperm)

fltByPerm(realdt, permdt)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pltMtfPrtIPL.R
\name{pltMtfPrtIPL}
\alias{pltMtfPrtIPL}
\title{Plot a heatmap of IPLs on proteins from consensus pathway submodules}
\usage{
pltMtfPrtIPL(fltmat, cldt, grphl, proteins = NULL)
}
\arguments{
\item{fltmat}{A matrix contains filterd IPL with rows as entity and
column as samples. This is the output from \code{fltByPerm()}.
Entity with NA value will be set to 0 and plotted as in
'normal' state.}

\item{cldt}{A data table with each row representing one clustering
result, and the first column denotes the number of
occurrences of a clustering result and the rest of columns
indicating each sample's cluster index. It is the output
of the \code{clSamp()} function. Only the most frequent
clustering result will be used to plot.}

\item{grphl}{A list of igraph objects representing consensus pathway
submodules. It is the output from \code{conMtf()}.}

\item{proteins}{A vector of proteins, of which IPLs to plot. Default: all
proteins that in both \code{grphl} and \code{fltmat}.}
}
\value{
A heatmap of IPLs o proteins from consensus pathway submodules
}
\description{
Plot a heatmap of IPLs on proteins from consensus pathway submodules
}
\examples{

fltmat <- system.file('extdata/pltSttKM/ipl.rds',package='MPAC') |> readRDS()
cldt <- system.file('extdata/pltMtfPrtIPL/cl.rds',package='MPAC')|> readRDS()
grphl <- system.file('extdata/pltMtfPrtIPL/grphl.rds',package='MPAC') |>
         readRDS()

pltMtfPrtIPL(fltmat, cldt, grphl, proteins=c('CD247', 'FASLG'))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pltOvrHm.R
\name{pltOvrHm}
\alias{pltOvrHm}
\title{Plot a heatmap of over-represented gene sets for clustered samples}
\usage{
pltOvrHm(ovrmat, cldt, min_frc = 0.8)
}
\arguments{
\item{ovrmat}{A matrix containing over-representation adjusted P with
rows as gene set names and columns as sample IDs. It is the
output of the \code{ovrGMT()} function.}

\item{cldt}{A data table with each row representing one clustering
result, and the first column denotes the number of
occurrences of a clustering result and the rest of columns
indicating each sample's cluster index. It is the output
of the \code{clSamp()} function. Only the most frequent
clustering result will be used to plot.}

\item{min_frc}{A minimum fraction of samples in a cluster that have a gene
set significantly over-represented (adjusted P < 0.05).
This is used to select gene sets to plot. Default: 0.8}
}
\value{
A heatmap with rows as over-represented gene sets and columns as
samples splited by clusters.
}
\description{
Plot a heatmap of over-represented gene sets for clustered samples
}
\examples{

ovrmat <- system.file('extdata/pltOvrHm/ovr.rds',package='MPAC') |> readRDS()
cldt   <- system.file('extdata/pltOvrHm/cl.rds', package='MPAC') |> readRDS()

pltOvrHm(ovrmat, cldt)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colIPL.R
\name{colRealIPL}
\alias{colRealIPL}
\title{Collect Inferred Pathway Levels (IPLs) from PARADIGM runs on real
data}
\usage{
colRealIPL(indir, sampleids = NULL, file_tag = NULL)
}
\arguments{
\item{indir}{Input folder that saves PARADIGM results. It should be set as
the same as \code{outdir} as in \code{runPrd()}.}

\item{sampleids}{Sample IDs for which IPLs to be collected. If not provided,
all files with suffix '_ipl.txt' in \code{indir} will be
collected. Default: NULL.}

\item{file_tag}{A string of output file name tag. Default: NULL}
}
\value{
A data.table object with columns of pathway entities and their IPLs.
}
\description{
Collect Inferred Pathway Levels (IPLs) from PARADIGM runs on real
data
}
\examples{

indir = system.file('/extdata/runPrd/', package='MPAC')

colRealIPL(indir)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runPrd.R
\name{runPrd}
\alias{runPrd}
\title{Run PARADIGM on multi-omic data}
\usage{
runPrd(real_se, fpth, outdir, PARADIGM_bin=NULL, nohup_bin=NULL,
    sampleids=NULL, file_tag=NULL, threads=1)
}
\arguments{
\item{real_se}{A SummarizedExperiment object of PARADIGM CNA and RNA states.
It is the same matrix as the output from \code{ppRealInp()}.}

\item{fpth}{Name of a pathway file for PARADIGM.}

\item{outdir}{Output folder to save all results.}

\item{PARADIGM_bin}{PARADIGM binary, which can be downloaded from
https://github.com/sng87/paradigm-scripts/tree/master/public/exe. Note that
the binary is only available for Linux or MacOS. Default: NULL}

\item{nohup_bin}{nohup binary, which is used for long running PARADIGM jobs.
Default: NULL}

\item{sampleids}{A vector of sample IDs to run PARADIGM on. If not provided,
all the samples that exist in both copy-number alteration
and RNA files will be ran. Default: NULL}

\item{file_tag}{A string of output file name tag. Default: NULL}

\item{threads}{Number of threads to run in parallel. Default: 1}
}
\value{
None
}
\description{
Run PARADIGM on multi-omic data
}
\examples{

freal = system.file('extdata/TcgaInp/inp_real.rds', package='MPAC')
real_se  = readRDS(freal)

fpth = system.file('extdata/Pth/tiny_pth.txt', package='MPAC')
outdir = tempdir()
paradigm_bin = '/path/to/PARADIGM'  ## change to binary location

# depends on external PARADIGM binary
runPrd(real_se, fpth, outdir, paradigm_bin, sampleids=c('TCGA-CV-7100'))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pltOvrHm.R
\name{getSignifOvrOnCl}
\alias{getSignifOvrOnCl}
\title{Get significantly over-represented gene sets for clustered samples}
\usage{
getSignifOvrOnCl(ovrmat, cldt, min_frc = 0.8)
}
\arguments{
\item{ovrmat}{A matrix containing over-representation adjusted P with
rows as gene set names and columns as sample IDs. It is the
output of the \code{ovrGMT()} function.}

\item{cldt}{A data table with each row representing one clustering
result, and the first column denotes the number of
occurrences of a clustering result and the rest of columns
indicating each sample's cluster index. It is the output
of the \code{clSamp()} function. Only the most frequent
clustering result will be used to plot.}

\item{min_frc}{A minimum fraction of samples in a cluster that have a gene
set significantly over-represented (adjusted P < 0.05).
This is used to select gene sets to plot. Default: 0.8}
}
\value{
A list of a matrix and a data.table object. The matrix has rows as
over-represented gene sets, columns as samples, and each cell
stores an adjusted P for over-representation. The data.table has
the clustering informations with samples in the same order as the
matrix's column.
}
\description{
Get significantly over-represented gene sets for clustered samples
}
\examples{

ovrmat <- system.file('extdata/pltOvrHm/ovr.rds',package='MPAC') |> readRDS()
cldt   <- system.file('extdata/pltOvrHm/cl.rds', package='MPAC') |> readRDS()

getSignifOvrOnCl(ovrmat, cldt)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pltSttKM.R
\name{pltSttKM}
\alias{pltSttKM}
\title{Plot a Kaplan-Meier curve for samples stratified by given
protein(s)' pathway states}
\usage{
pltSttKM(
  cdrmat,
  fltmat,
  event = "OS",
  time = "OS_days",
  proteins = NULL,
  strat_func = ">0"
)
}
\arguments{
\item{cdrmat}{A matrix containing survival data with rows as patient
samples and columns as survival event and time.}

\item{fltmat}{A matrix contains filterd IPL with rows as entity and
column as samples. This is the output from \code{fltByPerm()}.
Entity with NA value will be set to 0 and plotted as in
'normal' state.}

\item{event}{The column name in \code{cdrmat} to indicate survival event.
Default: 'OS'.}

\item{time}{The column name in \code{cdrmat} to indicate survival time.
Default: 'OS_days'.}

\item{proteins}{Rowname(s) in \code{fltmat}. Its/their pathway states will be
used to stratify patient samples. Default: all proteins
in \code{fltmat} will be used.}

\item{strat_func}{A function applied on protein(s) pathway states to
stratify patient samples. Available options: '>0', '<0',
Default: '>0', i.e., IPL >0 vs. the rest.}
}
\value{
A Kaplan-Meier plot
}
\description{
Plot a Kaplan-Meier curve for samples stratified by given
protein(s)' pathway states
}
\examples{

cdrmat <- system.file('extdata/pltSttKM/cdr.rds',package='MPAC') |> readRDS()
fltmat <- system.file('extdata/pltSttKM/ipl.rds',package='MPAC') |> readRDS()

pltSttKM(cdrmat, fltmat, event='OS', time='OS_days',
         proteins=c('CD247', 'FASLG'))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ppInp.R
\name{ppRnaInp}
\alias{ppRnaInp}
\title{Prepare input RNA data to run PARADIGM}
\usage{
ppRnaInp(rna_tumor_mat, rna_normal_mat, rna_n_sd = 2, threads = 1)
}
\arguments{
\item{rna_tumor_mat}{A matrix of RNA data from tumor samples with rows as
genes and columns as samples}

\item{rna_normal_mat}{A matrix of RNA data from normal samples with rows
as genes and columns as samples}

\item{rna_n_sd}{Standard deviation range from fitted normal samples to
define RNA state. Default: 2, i.e. 2*sd}

\item{threads}{Number of threads to run in parallel. Default: 1}
}
\value{
A SummarizedExperiment of RNA state for PARADIGM
}
\description{
Prepare input RNA data to run PARADIGM
}
\examples{

ftumor = system.file('extdata/TcgaInp/log10fpkmP1_tumor.rds', package='MPAC')
fnorm = system.file('extdata/TcgaInp/log10fpkmP1_normal.rds', package='MPAC')
rna_tumor_mat = readRDS(ftumor)
rna_norm_mat  = readRDS(fnorm)

ppRnaInp(rna_tumor_mat, rna_norm_mat, threads=2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ppInp.R
\name{ppCnInp}
\alias{ppCnInp}
\title{Prepare input copy-number (CN) alteration data to run PARADIGM}
\usage{
ppCnInp(cn_tumor_mat)
}
\arguments{
\item{cn_tumor_mat}{A matrix of tumor CN focal data with rows as genes
and columns as samples. A value of 0 means normal CN,
> 0 means amplification, and < 0 means deletion.}
}
\value{
A SummarizedExperiment object of CN state for PARADIGM
}
\description{
Prepare input copy-number (CN) alteration data to run PARADIGM
}
\examples{

fcn = system.file('extdata/TcgaInp/focal_tumor.rds', package='MPAC')
cn_tumor_mat = readRDS(fcn)

ppCnInp(cn_tumor_mat)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pltNeiStt.R
\name{pltNeiStt}
\alias{pltNeiStt}
\title{Plot a heatmap of pathway and omic states of a protein and its
pathway neighbors}
\usage{
pltNeiStt(real_se, fltmat, fpth, protein = "")
}
\arguments{
\item{real_se}{A SummarizedExperiment object of PARADIGM CNA and RNA
states. It is the output fromm \code{ppRealInp()} and must
contain the omic states for the one defined in the
\code{protein} argument.}

\item{fltmat}{A matrix contains filterd IPL with rows as entity and
column as samples. This is the output from \code{fltByPerm()}.
Entity with NA value will be set to 0 and plotted as in
'normal' state.}

\item{fpth}{Name of a pathway file for PARADIGM.}

\item{protein}{Name of the protein to plot. It requires to have CN and RNA
state data, as well as pathway data from the input.
Default: ''}
}
\value{
A heatmap of pathway and omic states of a protein and its
pathway neighbors
}
\description{
Plot a heatmap of pathway and omic states of a protein and its
pathway neighbors
}
\examples{

fpth = system.file('extdata/Pth/tiny_pth.txt', package='MPAC')

freal = system.file('extdata/pltNeiStt/inp_real.rds', package='MPAC')
fflt  = system.file('extdata/pltNeiStt/fltmat.rds',   package='MPAC')

real_se = readRDS(freal)
fltmat = readRDS(fflt)
protein = 'CD86'

pltNeiStt(real_se, fltmat, fpth, protein)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runPrd.R
\name{ppRunPrd}
\alias{ppRunPrd}
\title{Prepare required files to run PARADIGM}
\usage{
ppRunPrd(pat, cnmat, rnamat, outdir, file_tag=NULL)
}
\arguments{
\item{pat}{Sample ID}

\item{cnmat}{CN matrix}

\item{rnamat}{RNA matrix}

\item{outdir}{Output folder to save all results.}

\item{file_tag}{A string of output file name tag. Default: NULL}
}
\value{
None
}
\description{
Prepare required files to run PARADIGM
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clSamp.R
\name{clSamp}
\alias{clSamp}
\title{Cluster samples by pathway over-representation}
\usage{
clSamp(ovrmat, n_neighbors = 10, n_random_runs = 200, threads = 1)
}
\arguments{
\item{ovrmat}{A matrix of gene set over-representation adjusted p-values
with rows as gene sets and columns as samples. It is the
output from \code{ovrGMT()}.}

\item{n_neighbors}{Number of neighbors for clustering. A larger number is
recommended if the size of samples is large. Default: 10.}

\item{n_random_runs}{Number of random runs. Due to randomness introduced
to the Louvain algorithm in R igraph 1.3.0
(https://github.com/igraph/rigraph/issues/539), a large
number of runs are recommended to evaluate randomness
in the clustering results. Default: 200, which shall be
safe for sample size < 50. Please increase it
accordingly for a larger sample size.}

\item{threads}{Number of threads to run in parallel. Default: 1}
}
\value{
A data table with each row representing one clustering result, and
the first column denotes the number of occurrences of a clustering
result and the rest of columns indicating each sample's cluster
index. Rows are ordered by the number of occurrences from high to
low.
}
\description{
Cluster samples by pathway over-representation
}
\examples{

fovr = system.file('extdata/clSamp/ovrmat.rds', package='MPAC')
ovrmat = readRDS(fovr)

clSamp(ovrmat)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subNtw.R
\name{subNtw}
\alias{subNtw}
\title{Subset pathways by IPL results}
\usage{
subNtw(fltmat, fpth, fgmt, min_n_gmt_gns = 2, threads = 1)
}
\arguments{
\item{fltmat}{A matrix contains filterd IPL with rows
as 'entity' and column as samples. This is the output from
\code{fltByPerm()}. Entity with NA in all columns will be ignored.}

\item{fpth}{Name of a pathway file for PARADIGM.}

\item{fgmt}{A gene set GMT file. This will be the same file used for the
gene set over-representation calculation in the next step. It
is used here to ensure output sub-pathway contains a minimum
number of genes from to-be-used gene sets.}

\item{min_n_gmt_gns}{Minimum number of genes from the GMT file in the output
sub-pathway. Default: 2.}

\item{threads}{Number of threads to run in parallel. Default: 1}
}
\value{
A list of igraph objects representing the largest sub-pathway
for each sample.
}
\description{
Subset pathways by IPL results
}
\examples{

fflt = system.file('extdata/fltByPerm/flt_real.rds', package='MPAC')
fltmat = readRDS(fflt)
fpth = system.file('extdata/Pth/tiny_pth.txt',       package='MPAC')
fgmt = system.file('extdata/ovrGMT/fake.gmt',        package='MPAC')

subNtw(fltmat, fpth, fgmt, min_n_gmt_gns=1)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runPrd.R
\name{runPermPrd}
\alias{runPermPrd}
\title{Run PARADIGM on permuted data}
\usage{
runPermPrd(perml, fpth, outdir,
    PARADIGM_bin=NULL, nohup_bin=NULL, sampleids=NULL, threads=1)
}
\arguments{
\item{perml}{A list of SummarizedExperiment objects of permuted CNA and
RNA states generated by \code{ppPermInp()}.}

\item{fpth}{Name of a pathway file for PARADIGM.}

\item{outdir}{Output folder to save all results.}

\item{PARADIGM_bin}{PARADIGM binary, which can be downloaded from
https://github.com/sng87/paradigm-scripts/tree/master/public/exe. Note that
the binary is only available for Linux or MacOS. Default: NULL}

\item{nohup_bin}{nohup binary, which is used for long running PARADIGM jobs.
Default: NULL}

\item{sampleids}{A vector of sample IDs to run PARADIGM on. If not provided,
all the samples that exist in both copy-number alteration
and RNA files will be ran. Default: NULL}

\item{threads}{Number of threads to run in parallel. Default: 1}
}
\value{
None
}
\description{
Run PARADIGM on permuted data
}
\examples{

fperm = system.file('extdata/TcgaInp/inp_perm.rds', package='MPAC')
perml = readRDS(fperm)
fpth = system.file('extdata/Pth/tiny_pth.txt', package='MPAC')
outdir = tempdir()
paradigm_bin = '/path/to/PARADIGM'  ## change to binary location
pat = 'TCGA-CV-7100'

# depends on external PARADIGM binary, do not run
runPermPrd(perml, fpth, outdir, paradigm_bin, sampleids=c(pat))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conMtf.R
\name{conMtf}
\alias{conMtf}
\title{Find consensus pathway motifs from a list of pathways}
\usage{
conMtf(subntwl, omic_genes = NULL, min_mtf_n_nodes = 5)
}
\arguments{
\item{subntwl}{A list of igraph objects representing input pathways from
different samples. It is the output from \code{subNtw()}}

\item{omic_genes}{A vector of gene symbols to narrow down
over-representation calculation to only those with input
genomic data. If not provided, all genes in the GMT file
will be considered. Default: NULL.}

\item{min_mtf_n_nodes}{Number of minimum nodes in a motif. Default: 5}
}
\value{
A list of igraph objects representing consensus pathway motifs
}
\description{
Find consensus pathway motifs from a list of pathways
}
\examples{

fsubntwl = system.file('extdata/conMtf/subntwl.rds', package='MPAC')
subntwl = readRDS(fsubntwl)

fomic_gns = system.file('extdata/TcgaInp/inp_focal.rds', package='MPAC')
omic_gns = rownames(readRDS(fomic_gns))

conMtf(subntwl, omic_gns, min_mtf_n_nodes=50)

}
